@@ -168,7 +168,6 @@ protected:
 
 txLoadListenerProxy::txLoadListenerProxy(nsWeakPtr aParent)
 {
-    NS_INIT_ISUPPORTS();
     mParent = aParent;
 }
 
@@ -240,7 +239,6 @@ txLoadListenerProxy::Error(nsIDOMEvent* aEvent)
 
 nsSyncLoader::nsSyncLoader()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsSyncLoader::~nsSyncLoader()
@@ -294,11 +292,20 @@ nsSyncLoader::LoadDocument(nsIChannel* aChannel,
     nsCOMPtr<nsIDocument> document = do_CreateInstance(kXMLDocumentCID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
+    // Start the document load. Do this before we attach the load listener
+    // since we reset the document which drops all observers.
+    nsCOMPtr<nsIStreamListener> listener;
+    rv = document->StartDocumentLoad(kLoadAsData, mChannel, 
+                                     loadGroup, nsnull, 
+                                     getter_AddRefs(listener),
+                                     PR_TRUE);
+    NS_ENSURE_SUCCESS(rv, rv);
+
     // Register as a load listener on the document
     nsCOMPtr<nsIDOMEventReceiver> target = do_QueryInterface(document);
     NS_ENSURE_TRUE(target, NS_ERROR_FAILURE);
 
-    nsWeakPtr requestWeak = getter_AddRefs(NS_GetWeakReference(NS_STATIC_CAST(nsIDOMLoadListener*, this)));
+    nsWeakPtr requestWeak = do_GetWeakReference(NS_STATIC_CAST(nsIDOMLoadListener*, this));
     txLoadListenerProxy* proxy = new txLoadListenerProxy(requestWeak);
     NS_ENSURE_TRUE(proxy, NS_ERROR_OUT_OF_MEMORY);
 
@@ -309,19 +316,6 @@ nsSyncLoader::LoadDocument(nsIChannel* aChannel,
     NS_ENSURE_SUCCESS(rv, rv);
     
     mLoadSuccess = PR_FALSE;
-
-    nsCOMPtr<nsIStreamListener> listener;
-    rv = document->StartDocumentLoad(kLoadAsData, mChannel, 
-                                     loadGroup, nsnull, 
-                                     getter_AddRefs(listener),
-                                     PR_FALSE);
-    if (NS_FAILED(rv)) {
-        target->RemoveEventListenerByIID(NS_STATIC_CAST(nsIDOMEventListener*, 
-                                                        proxy), 
-                                         NS_GET_IID(nsIDOMLoadListener));
-        return rv;
-    }
-
     if (aChannelIsSync) {
         rv = PushSyncStream(listener);
     }
@@ -396,6 +390,7 @@ nsSyncLoader::PushSyncStream(nsIStreamListener* aListener)
     nsresult rv = mChannel->Open(getter_AddRefs(in));
     NS_ENSURE_SUCCESS(rv, rv);
 
+    mLoading = PR_TRUE;
     rv = nsSyncLoadService::PushSyncStreamToListener(in, aListener, mChannel);
 
     return rv;
@@ -484,7 +479,6 @@ nsSyncLoader::GetInterface(const nsIID & aIID,
 
 nsSyncLoadService::nsSyncLoadService()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsSyncLoadService::~nsSyncLoadService()
@@ -544,10 +538,6 @@ nsSyncLoadService::LoadLocalXBLDocument(nsIChannel* aChannel,
     nsCOMPtr<nsIDocument> doc = do_CreateInstance(kXMLDocumentCID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    // Do this after making sure the |channel->Open| succeeded (which it
-    // won't if the file doesn't exist) so that we don't have to go
-    // through the work of breaking the circular references between
-    // content sink, script loader, and document.
     nsCOMPtr<nsIXMLContentSink> xblSink;
     rv = NS_NewXBLContentSink(getter_AddRefs(xblSink), doc, docURI, nsnull);
     NS_ENSURE_SUCCESS(rv, rv);
