@@ -43,11 +43,9 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsStyleUtil.h"
 #include "nsIDocument.h"
 #include "nsIHTMLDocument.h"
 #include "nsIHTMLStyleSheet.h"
@@ -81,13 +79,8 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD Equals(const nsIStyleRule* aRule, PRBool& aResult) const;
-  NS_IMETHOD HashValue(PRUint32& aValue) const;
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
 
-  // Strength is an out-of-band weighting, always 0 here
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
-
   // The new mapping function.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
 
@@ -133,7 +126,7 @@ public:
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
+                                      nsChangeHint& aHint) const;
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
@@ -146,7 +139,6 @@ protected:
 
 BodyRule::BodyRule(nsHTMLBodyElement* aPart, nsIHTMLStyleSheet* aSheet)
 {
-  NS_INIT_REFCNT();
   mPart = aPart;
   mSheet = aSheet;
 }
@@ -157,20 +149,6 @@ BodyRule::~BodyRule()
 
 NS_IMPL_ISUPPORTS1(BodyRule, nsIStyleRule)
 
-NS_IMETHODIMP
-BodyRule::Equals(const nsIStyleRule* aRule, PRBool& aResult) const
-{
-  aResult = PRBool(this == aRule);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-BodyRule::HashValue(PRUint32& aValue) const
-{
-  aValue = NS_PTR_TO_INT32(mPart);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 BodyRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
 {
@@ -179,15 +157,6 @@ BodyRule::GetStyleSheet(nsIStyleSheet*& aSheet) const
   return NS_OK;
 }
 
-// Strength is an out-of-band weighting, useful for mapping CSS !
-// important always 0 here
-NS_IMETHODIMP
-BodyRule::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 BodyRule::MapRuleInfoInto(nsRuleData* aData)
 {
@@ -337,7 +306,7 @@ void BodyRule::SizeOf(nsISizeOfHandler *aSizeOfHandler, PRUint32 &aSize)
 
   // create a tag for this instance
   nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("BodyRule"));
+  tag = do_GetAtom("BodyRule");
   // get the size of an empty instance and add to the sizeof handler
   aSize = sizeof(*this);
   aSizeOfHandler->AddSize(tag, aSize);
@@ -437,129 +406,52 @@ nsHTMLBodyElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 
 NS_IMPL_STRING_ATTR(nsHTMLBodyElement, Background, background)
 
-
-NS_IMETHODIMP
-nsHTMLBodyElement::GetVLink(nsAString& aVlinkColor)
-{
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::vlink, aVlinkColor);
-
-  // If we don't have an attribute, find the default color from the
-  // UA stylesheet.
-  if (rv == NS_CONTENT_ATTR_NOT_THERE) {
-    nsCOMPtr<nsIPresContext> context;
-    GetPresContext(this, getter_AddRefs(context));
-
-    if (context) {
-      nscolor vlinkColor;
-      context->GetDefaultVisitedLinkColor(&vlinkColor);
-
-      nsHTMLValue value(vlinkColor);
-      ColorToString(value, aVlinkColor);
-    }
-  }
-
-  return NS_OK;
+#define NS_IMPL_HTMLBODY_COLOR_ATTR(attr_, func_, default_)         \
+NS_IMETHODIMP                                                       \
+nsHTMLBodyElement::Get##func_(nsAString& aColor)                    \
+{                                                                   \
+  aColor.Truncate();                                                \
+  nsAutoString color;                                               \
+  nscolor attrColor;                                                \
+  if (NS_CONTENT_ATTR_NOT_THERE ==                                  \
+      GetAttr(kNameSpaceID_None, nsHTMLAtoms::attr_, color)) {      \
+                                                                    \
+    nsCOMPtr<nsIPresContext> presContext;                           \
+    GetPresContext(this, getter_AddRefs(presContext));              \
+                                                                    \
+    if (presContext) {                                              \
+      presContext->GetDefault##default_(&attrColor);                \
+      nsHTMLValue(attrColor).ToString(aColor);                      \
+    }                                                               \
+  } else if (NS_ColorNameToRGB(color, &attrColor)) {                \
+    nsHTMLValue(attrColor).ToString(aColor);                        \
+  } else {                                                          \
+    aColor.Assign(color);                                           \
+  }                                                                 \
+  return NS_OK;                                                     \
+}                                                                   \
+NS_IMETHODIMP                                                       \
+nsHTMLBodyElement::Set##func_(const nsAString& aColor)              \
+{                                                                   \
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::attr_, aColor,     \
+                 PR_TRUE);                                          \
 }
 
-NS_IMETHODIMP
-nsHTMLBodyElement::SetVLink(const nsAString& aVlinkColor)
-{
-  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::vlink, aVlinkColor, PR_TRUE);
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::GetALink(nsAString& aAlinkColor)
-{
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::alink, aAlinkColor);
-
-  // If we don't have an attribute, find the default color from the
-  // UA stylesheet.
-  if (rv == NS_CONTENT_ATTR_NOT_THERE) {
-    nsCOMPtr<nsIPresContext> context;
-    GetPresContext(this, getter_AddRefs(context));
-
-    if (context) {
-      // XXX We don't have the backend or the UI to get ALINKs from the
-      // UA stylesheet yet, so we'll piggyback to the default link color like IE.
-      nscolor alinkColor;
-      context->GetDefaultLinkColor(&alinkColor);
-
-      nsHTMLValue value(alinkColor);
-      ColorToString(value, aAlinkColor);
-    }
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::SetALink(const nsAString& aAlinkColor)
-{
-  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::alink, aAlinkColor, PR_TRUE);
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::GetLink(nsAString& aLinkColor)
-{
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::link, aLinkColor);
-
-  // If we don't have an attribute, find the default color from the
-  // UA stylesheet.
-  if (rv == NS_CONTENT_ATTR_NOT_THERE) {
-    nsCOMPtr<nsIPresContext> context;
-    GetPresContext(this, getter_AddRefs(context));
-
-    if (context) {
-      nscolor linkColor;
-      context->GetDefaultLinkColor(&linkColor);
-
-      nsHTMLValue value(linkColor);
-      ColorToString(value, aLinkColor);
-    }
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::SetLink(const nsAString& aLinkColor)
-{
-  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::link, aLinkColor, PR_TRUE);
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::GetText(nsAString& aTextColor)
-{
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::text, aTextColor);
-
-  // If we don't have an attribute, find the default color from the
-  // UA stylesheet.
-  if (rv == NS_CONTENT_ATTR_NOT_THERE) {
-    nsCOMPtr<nsIPresContext> context;
-    GetPresContext(this, getter_AddRefs(context));
-
-    if (context) {
-      nscolor textColor;
-      context->GetDefaultColor(&textColor);
-
-      nsHTMLValue value(textColor);
-      ColorToString(value, aTextColor);
-    }
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLBodyElement::SetText(const nsAString& aTextColor)
-{
-  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::text, aTextColor, PR_TRUE);
-}
+NS_IMPL_HTMLBODY_COLOR_ATTR(vlink, VLink, VisitedLinkColor)
+NS_IMPL_HTMLBODY_COLOR_ATTR(alink, ALink, LinkColor)
+NS_IMPL_HTMLBODY_COLOR_ATTR(link, Link, LinkColor)
+// XXX Should text check the body frame's style struct for color,
+// like we do for bgColor?
+NS_IMPL_HTMLBODY_COLOR_ATTR(text, Text, Color)
 
 NS_IMETHODIMP 
 nsHTMLBodyElement::GetBgColor(nsAString& aBgColor)
 {
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::bgcolor, aBgColor);
+  aBgColor.Truncate();
+
+  nsAutoString attr;
+  nscolor bgcolor;
+  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::bgcolor, attr);
 
   // If we don't have an attribute, find the actual color used for
   // (generally from the user agent style sheet) for compatibility
@@ -589,11 +481,20 @@ nsHTMLBodyElement::GetBgColor(nsAString& aBgColor)
                                    (const nsStyleStruct*&)StyleBackground);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        nsHTMLValue value(StyleBackground->mBackgroundColor);
-        ColorToString(value, aBgColor);
+        bgcolor = StyleBackground->mBackgroundColor;
+        nsHTMLValue(bgcolor).ToString(aBgColor);
       }
     }
   }
+  else if (NS_ColorNameToRGB(attr, &bgcolor)) {
+    // If we have a color name which we can convert to an nscolor,
+    // then we should use the hex value instead of the color name.
+    nsHTMLValue(bgcolor).ToString(aBgColor);
+  }
+  else {
+    // Otherwise, just assign whatever the attribute value is.
+    aBgColor.Assign(attr);
+  }
 
   return NS_OK;
 }
@@ -614,7 +515,7 @@ nsHTMLBodyElement::StringToAttribute(nsIAtom* aAttribute,
       (aAttribute == nsHTMLAtoms::link) ||
       (aAttribute == nsHTMLAtoms::alink) ||
       (aAttribute == nsHTMLAtoms::vlink)) {
-    if (ParseColor(aValue, mDocument, aResult)) {
+    if (aResult.ParseColor(aValue, mDocument)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -622,7 +523,7 @@ nsHTMLBodyElement::StringToAttribute(nsIAtom* aAttribute,
            (aAttribute == nsHTMLAtoms::marginheight) ||
            (aAttribute == nsHTMLAtoms::topmargin) ||
            (aAttribute == nsHTMLAtoms::leftmargin)) {
-    if (ParseValue(aValue, 0, aResult, eHTMLUnit_Pixel)) {
+    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Pixel, 0)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -661,11 +562,12 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
       nsCOMPtr<nsIDocument> doc;
       presShell->GetDocument(getter_AddRefs(doc));
       if (doc) {
-        nsIHTMLContentContainer*  htmlContainer;
-        if (NS_OK == doc->QueryInterface(NS_GET_IID(nsIHTMLContentContainer),
-                                         (void**)&htmlContainer)) {
-          nsIHTMLStyleSheet* styleSheet;
-          if (NS_OK == htmlContainer->GetAttributeStyleSheet(&styleSheet)) {
+        nsCOMPtr<nsIHTMLContentContainer> htmlContainer =
+          do_QueryInterface(doc);
+        if (htmlContainer) {
+          nsCOMPtr<nsIHTMLStyleSheet> styleSheet;
+          htmlContainer->GetAttributeStyleSheet(getter_AddRefs(styleSheet));
+          if (styleSheet) {
             aAttributes->GetAttribute(nsHTMLAtoms::link, value);
             if ((eHTMLUnit_Color == value.GetUnit()) || 
                 (eHTMLUnit_ColorName == value.GetUnit())) {
@@ -683,10 +585,7 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
                 (eHTMLUnit_ColorName == value.GetUnit())) {
               styleSheet->SetVisitedLinkColor(value.GetColorValue());
             }
-
-            NS_RELEASE(styleSheet);
           }
-          NS_RELEASE(htmlContainer);
         }
       }
     }
@@ -753,7 +652,7 @@ nsHTMLBodyElement::WalkContentStyleRules(nsRuleWalker* aRuleWalker)
 
 NS_IMETHODIMP
 nsHTMLBodyElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                            PRInt32& aHint) const
+                                            nsChangeHint& aHint) const
 {
   if ((aAttribute == nsHTMLAtoms::link) ||
       (aAttribute == nsHTMLAtoms::vlink) ||
