@@ -48,7 +48,6 @@
 #include "nsISupportsArray.h"
 #include "nsCRT.h"
 #include "nsIArena.h"
-#include "nsIStyleContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsIHTMLContent.h"
 #include "nsVoidArray.h"
@@ -353,8 +352,6 @@ public:
   // nsIStyleRule 
   NS_IMETHOD GetStyleSheet(nsIStyleSheet*& aSheet) const;
   NS_IMETHOD SetStyleSheet(nsIHTMLStyleSheet* aSheet);
-  // Strength is an out-of-band weighting, always 0 here
-  NS_IMETHOD GetStrength(PRInt32& aStrength) const;
   
   // The new mapping functions.
   NS_IMETHOD MapRuleInfoInto(nsRuleData* aRuleData);
@@ -381,7 +378,6 @@ nsHTMLMappedAttributes::nsHTMLMappedAttributes(void)
     mRuleMapper(nsnull),
     mUniqued(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsHTMLMappedAttributes::nsHTMLMappedAttributes(const nsHTMLMappedAttributes& aCopy)
@@ -392,7 +388,6 @@ nsHTMLMappedAttributes::nsHTMLMappedAttributes(const nsHTMLMappedAttributes& aCo
     mRuleMapper(aCopy.mRuleMapper),
     mUniqued(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 
   HTMLAttribute::CopyHTMLAttributes(aCopy.mFirst.mNext, &(mFirst.mNext));
 }
@@ -770,13 +765,6 @@ nsHTMLMappedAttributes::SetStyleSheet(nsIHTMLStyleSheet* aSheet)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLMappedAttributes::GetStrength(PRInt32& aStrength) const
-{
-  aStrength = 0;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsHTMLMappedAttributes::MapRuleInfoInto(nsRuleData* aRuleData)
 {
@@ -828,7 +816,7 @@ void nsHTMLMappedAttributes::SizeOf(nsISizeOfHandler* aSizer, PRUint32 &aResult)
     return;
   }
   nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("HTMLMappedAttributes"));
+  tag = do_GetAtom("HTMLMappedAttributes");
   aResult = sizeof(*this);
   aSizer->AddSize(tag, aResult);
 }
@@ -1323,7 +1311,7 @@ nsHTMLAttributes::HasAttribute(nsIAtom* aAttrName, PRInt32 aNamespaceID) const
       mMapped->HasAttribute(aAttrName))
     return PR_TRUE;
 
-  return (PRBool)HTMLAttribute::FindHTMLAttribute(aAttrName, aNamespaceID, mFirstUnmapped);
+  return (PRBool)(HTMLAttribute::FindHTMLAttribute(aAttrName, aNamespaceID, mFirstUnmapped) != nsnull);
 }
 
 NS_IMETHODIMP
@@ -1442,7 +1430,7 @@ nsHTMLAttributes::GetClasses(nsVoidArray& aArray) const
   return NS_OK;
 }
 
-NS_IMETHODIMP
+NS_IMETHODIMP_(PRBool)
 nsHTMLAttributes::HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const
 {
   NS_PRECONDITION(aClass, "unexpected null pointer");
@@ -1451,7 +1439,7 @@ nsHTMLAttributes::HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const
     if (aCaseSensitive) {
       do {
         if (classList->mAtom == aClass)
-          return NS_OK;
+          return PR_TRUE;
         classList = classList->mNext;
       } while (classList);
     } else {
@@ -1466,12 +1454,12 @@ nsHTMLAttributes::HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const
         classList->mAtom->GetUnicode(&class2Buf);
         nsDependentString class2(class2Buf);
         if (class1.Equals(class2, nsCaseInsensitiveStringComparator()))
-          return NS_OK;
+          return PR_TRUE;
         classList = classList->mNext;
       } while (classList);
     }
   }
-  return NS_COMFALSE;
+  return PR_FALSE;
 }
 
 #ifdef UNIQUE_ATTR_SUPPORT
@@ -1634,7 +1622,7 @@ void nsHTMLAttributes::SizeOf(nsISizeOfHandler* aSizer, PRUint32 &aResult)
   aResult = sum;
 
   nsCOMPtr<nsIAtom> tag;
-  tag = getter_AddRefs(NS_NewAtom("nsHTMLAttributes"));
+  tag = do_GetAtom("nsHTMLAttributes");
   aSizer->AddSize(tag, aResult);
 }
 #endif
