@@ -13,7 +13,7 @@
  * Portions created by ActiveState Tool Corp. are Copyright (C) 2000, 2001
  * ActiveState Tool Corp.  All Rights Reserved.
  *
- * Contributor(s): Mark Hammond <MarkH@ActiveState.com> (original author)
+ * Contributor(s): Mark Hammond <mhammond@skippinet.com.au> (original author)
  *
  */
 
@@ -60,6 +60,9 @@ extern nsIID Py_nsIID_NULL;
 **************************************************************************
 *************************************************************************/
 
+#define NS_PYXPCOM_NO_SUCH_METHOD \
+  NS_ERROR_GENERATE_SUCCESS(NS_ERROR_MODULE_PYXPCOM, 0)
+
 // The exception object (loaded from the xpcom .py code)
 extern PYXPCOM_EXPORT PyObject *PyXPCOM_Error;
 
@@ -176,9 +179,14 @@ public:
 	//  see the correct xpcom.client.Interface object even when calling
 	//  xpcom function directly.
 	static PyObject *PyObjectFromInterface(nsISupports *ps, 
-		                               const nsIID &iid, 
-					       PRBool bAddRef,
-					       PRBool bMakeNicePyObject = PR_TRUE);
+	                                       const nsIID &iid, 
+	                                       PRBool bAddRef,
+	                                       PRBool bMakeNicePyObject = PR_TRUE);
+
+	static PyObject *PyObjectFromInterfaceOrVariant(nsISupports *ps,
+	                                                const nsIID &iid, 
+	                                                PRBool bAddRef,
+	                                                PRBool bMakeNicePyObject = PR_TRUE);
 
 	// Given a Python object that is a registered COM type, return a given
 	// interface pointer on its underlying object, with a NEW REFERENCE ADDED.
@@ -187,6 +195,8 @@ public:
 	//  provided to stop accidental recursion should the object returned by
 	//  the wrap process itself be in instance (where it should already be
 	//  a COM object.
+	// If |iid|==nsIVariant, then arbitary Python objects will be wrapped
+	// in an nsIVariant.
 	static PRBool InterfaceFromPyObject(
 		PyObject *ob,
 		const nsIID &iid,
@@ -194,6 +204,13 @@ public:
 		PRBool bNoneOK,
 		PRBool bTryAutoWrap = PR_TRUE);
 
+	// Given a Py_nsISupports, return an interface.
+	// Object *must* be Py_nsISupports - there is no
+	// "autowrap", no "None" support, etc
+	static PRBool InterfaceFromPyISupports(PyObject *ob, 
+	                                       const nsIID &iid, 
+	                                       nsISupports **ppv);
+
 	static Py_nsISupports *Constructor(nsISupports *pInitObj, const nsIID &iid);
 	// The Python methods
 	static PyObject *QueryInterface(PyObject *self, PyObject *args);
@@ -307,12 +324,12 @@ class PyXPCOM_GatewayWeakReference;
 // This interface is needed primarily to give us a known vtable base.
 // If we QI a Python object for this interface, we can safely cast the result
 // to a PyG_Base.  Any other interface, we do now know which vtable we will get.
-// Later, we may get some internal functions
-// (eg, win32com allows us to get the underlying Python object, but
-// we should try and avoid that if possible.
+// We also allow the underlying PyObject to be extracted
 class nsIInternalPython : public nsISupports {
- public: 
-  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IINTERNALPYTHON_IID)
+public: 
+	NS_DEFINE_STATIC_IID_ACCESSOR(NS_IINTERNALPYTHON_IID)
+	// Get the underlying Python object with new reference added
+	virtual PyObject *UnwrapPythonObject(void) = 0;
 };
 
 // This is roughly equivilent to PyGatewayBase in win32com
@@ -322,6 +339,7 @@ class PYXPCOM_EXPORT PyG_Base : public nsIInternalPython, public nsISupportsWeak
 public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSISUPPORTSWEAKREFERENCE
+	PyObject *UnwrapPythonObject(void);
 
 	// A static "constructor" - the real ctor is protected.
 	static nsresult CreateNew(PyObject *pPyInstance, 
@@ -350,6 +368,7 @@ public:
 	// Helpers for "native" interfaces.
 	// Not used by the generic stub interface.
 	nsresult HandleNativeGatewayError(const char *szMethodName);
+
 	// These data members used by the converter helper functions - hence public
 	nsIID m_iid;
 	PyObject * m_pPyObject;
@@ -359,6 +378,9 @@ public:
 	// This means that once we have created it (and while we
 	// are alive) it will never die.
 	nsCOMPtr<nsIWeakReference> m_pWeakRef;
+#ifdef NS_BUILD_REFCNT_LOGGING
+	char refcntLogRepr[64]; // sigh - I wish I knew how to use the Moz string classes :(  OK for debug only tho.
+#endif
 protected:
 	PyG_Base(PyObject *instance, const nsIID &iid);
 	virtual ~PyG_Base();
@@ -428,6 +450,9 @@ public:
 	NS_DECL_ISUPPORTS
     NS_DECL_NSIWEAKREFERENCE;
 	PyG_Base *m_pBase; // NO REF COUNT!!!
+#ifdef NS_BUILD_REFCNT_LOGGING
+	char refcntLogRepr[41];
+#endif
 };
 
 
@@ -451,6 +476,7 @@ private:
 	PRUint32 GetSizeIs( int var_index, PRBool is_arg1);
 	PRBool SetSizeIs( int var_index, PRBool is_arg1, PRUint32 new_size);
 	PRBool CanSetSizeIs( int var_index, PRBool is_arg1 );
+	nsIInterfaceInfo *GetInterfaceInfo(); // NOTE: no ref count on result.
 
 
 	nsXPTCMiniVariant* m_params;
@@ -458,15 +484,20 @@ private:
 	int m_method_index;
 	PythonTypeDescriptor *m_python_type_desc_array;
 	int m_num_type_descs;
-
+	nsCOMPtr<nsIInterfaceInfo> m_interface_info;
 };
 
 // Misc converters.
+PyObject *PyObject_FromXPTType( const nsXPTType *d);
+// XPTTypeDescriptor derived from XPTType - latter is automatically processed via PyObject_FromXPTTypeDescriptor XPTTypeDescriptor 
 PyObject *PyObject_FromXPTTypeDescriptor( const XPTTypeDescriptor *d);
+
 PyObject *PyObject_FromXPTParamDescriptor( const XPTParamDescriptor *d);
 PyObject *PyObject_FromXPTMethodDescriptor( const XPTMethodDescriptor *d);
 PyObject *PyObject_FromXPTConstant( const XPTConstDescriptor *d);
 
+nsIVariant *PyObject_AsVariant( PyObject *ob);
+PyObject *PyObject_FromVariant( nsIVariant *v);
 
 // DLL reference counting functions.
 // Although we maintain the count, we never actually
@@ -650,7 +681,7 @@ PyXPCOM_TypeObject *ClassName::type = NULL;
 
 
 // And the classes
-PyXPCOM_INTERFACE_DECLARE(Py_nsIComponentManager, nsIComponentManager, PyMethods_IComponentManager)
+PyXPCOM_INTERFACE_DECLARE(Py_nsIComponentManager, nsIComponentManagerObsolete, PyMethods_IComponentManager)
 PyXPCOM_INTERFACE_DECLARE(Py_nsIInterfaceInfoManager, nsIInterfaceInfoManager, PyMethods_IInterfaceInfoManager)
 PyXPCOM_INTERFACE_DECLARE(Py_nsIEnumerator, nsIEnumerator, PyMethods_IEnumerator)
 PyXPCOM_INTERFACE_DECLARE(Py_nsISimpleEnumerator, nsISimpleEnumerator, PyMethods_ISimpleEnumerator)
@@ -658,5 +689,6 @@ PyXPCOM_INTERFACE_DECLARE(Py_nsIInterfaceInfo, nsIInterfaceInfo, PyMethods_IInte
 PyXPCOM_INTERFACE_DECLARE(Py_nsIServiceManager, nsIServiceManager, PyMethods_IServiceManager)
 PyXPCOM_INTERFACE_DECLARE(Py_nsIInputStream, nsIInputStream, PyMethods_IInputStream)
 PyXPCOM_ATTR_INTERFACE_DECLARE(Py_nsIClassInfo, nsIClassInfo, PyMethods_IClassInfo)
+PyXPCOM_ATTR_INTERFACE_DECLARE(Py_nsIVariant, nsIVariant, PyMethods_IVariant)
 
 #endif // __PYXPCOM_H__
