@@ -25,9 +25,9 @@ ExprResult* CurrentFunctionCall::evaluate(txIEvalContext* aContext)
     return new NodeSet(mPs->getEvalContext()->getContextNode());
 }
 
-nsresult CurrentFunctionCall::getNameAtom(txAtom** aAtom)
+nsresult CurrentFunctionCall::getNameAtom(nsIAtom** aAtom)
 {
     *aAtom = txXSLTAtoms::current;
-    TX_ADDREF_ATOM(*aAtom);
+    NS_ADDREF(*aAtom);
     return NS_OK;
 }
