@@ -138,8 +138,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsRenderingContextPS, nsIRenderingContext)
  */
 nsRenderingContextPS :: nsRenderingContextPS()
 {
-  NS_INIT_REFCNT();
-
   mPSObj = nsnull;     // local copy of printcontext, will be set on the init process
   mContext = nsnull;
   mFontMetrics = nsnull;
@@ -587,7 +585,7 @@ nsRenderingContextPS :: GetCurrentTransform(nsTransform2D *&aTransform)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: CreateDrawingSurface(nsRect *aBounds, PRUint32 aSurfFlags, nsDrawingSurface &aSurface)
+nsRenderingContextPS :: CreateDrawingSurface(const nsRect& aBounds, PRUint32 aSurfFlags, nsDrawingSurface &aSurface)
 {
   return NS_OK;   // offscreen test
 }
@@ -1023,6 +1021,35 @@ nsRenderingContextPS :: GetWidth(const PRUnichar *aString,PRUint32 aLength,nscoo
 }
 
 /** --------------------------------------------------- */
+NS_IMETHODIMP
+nsRenderingContextPS::GetTextDimensions(const char*       aString,
+                                        PRInt32           aLength,
+                                        PRInt32           aAvailWidth,
+                                        PRInt32*          aBreaks,
+                                        PRInt32           aNumBreaks,
+                                        nsTextDimensions& aDimensions,
+                                        PRInt32&          aNumCharsFit,
+                                        nsTextDimensions& aLastWordDimensions,
+                                        PRInt32*          aFontID)
+{
+  NS_NOTYETIMPLEMENTED("nsRenderingContextPS::GetTextDimensions");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsRenderingContextPS::GetTextDimensions(const PRUnichar*  aString,
+                                        PRInt32           aLength,
+                                        PRInt32           aAvailWidth,
+                                        PRInt32*          aBreaks,
+                                        PRInt32           aNumBreaks,
+                                        nsTextDimensions& aDimensions,
+                                        PRInt32&          aNumCharsFit,
+                                        nsTextDimensions& aLastWordDimensions,
+                                        PRInt32*          aFontID)
+{
+  NS_NOTYETIMPLEMENTED("nsRenderingContextPS::GetTextDimensions");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 NS_IMETHODIMP
 nsRenderingContextPS :: GetTextDimensions(const char* aString, PRUint32 aLength,
@@ -1063,51 +1090,47 @@ nsRenderingContextPS :: GetTextDimensions(const PRUnichar* aString, PRUint32 aLe
  *  See documentation in nsIRenderingContext.h
  *	@update 12/21/98 dwc
  */
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsRenderingContextPS :: DrawString(const char *aString, PRUint32 aLength,
                         nscoord aX, nscoord aY,
                         const nscoord* aSpacing)
 {
-PRInt32	      x = aX;
-PRInt32       y = aY;
+  NS_ENSURE_TRUE(mTranMatrix && mPSObj && mFontMetrics, NS_ERROR_NULL_POINTER);
 
-  mPSObj->setlanggroup(nsnull);
+  nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
+  NS_ENSURE_TRUE(metrics, NS_ERROR_FAILURE);
 
-  SetupFontAndColor();
-  PRInt32 dxMem[500];
-  PRInt32* dx0 = 0;
-  if (nsnull != aSpacing) {
-    dx0 = dxMem;
-    if (aLength > 500) {
-      dx0 = new PRInt32[aLength];
-    }
-    mTranMatrix->ScaleXCoords(aSpacing, aLength, dx0);
-  }
+  nsCOMPtr<nsIAtom> langGroup;
+  mFontMetrics->GetLangGroup(getter_AddRefs(langGroup));
+  mPSObj->setlanggroup(langGroup);
 
-	mTranMatrix->TransformCoord(&x, &y);
-  PostscriptTextOut(aString, aLength, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aLength, (const nscoord*) (aSpacing ? dx0 : nsnull), PR_FALSE);
+  if (aLength == 0)
+    return NS_OK;
+  nsFontPS* fontPS = nsFontPS::FindFont(aString[0], *metrics->GetFont(), metrics);
+  NS_ENSURE_TRUE(fontPS, NS_ERROR_FAILURE);
+  fontPS->SetupFont(this);
 
-  if ((nsnull != aSpacing) && (dx0 != dxMem)) {
-    delete [] dx0;
-  }
+  PRUint32 i, start = 0;
+  for (i=0; i<aLength; i++) {
+    nsFontPS* fontThisChar;
+    fontThisChar = nsFontPS::FindFont(aString[i], *metrics->GetFont(), metrics);
+    NS_ENSURE_TRUE(fontThisChar, NS_ERROR_FAILURE);
+    if (fontThisChar != fontPS) {
+      // draw text up to this point
+      aX += DrawString(aString+start, i-start, aX, aY, fontPS, 
+                       aSpacing?aSpacing+start:nsnull);
+      start = i;
 
-#if 0
-  //this doesn't need to happen here anymore, but a
-  //new api will come along that will need this stuff. MMP
-  if (mFontMetrics) {
-    nsFont *font;
-    mFontMetrics->GetFont(font);
-    PRUint8 decorations = font->decorations;
-
-    if (decorations & NS_FONT_DECORATION_OVERLINE){
-      nscoord offset;
-      nscoord size;
-      mFontMetrics->GetUnderline(offset, size);
-      FillRect(aX, aY, aWidth, size);
+      // setup for following text
+      fontPS = fontThisChar;
+      fontPS->SetupFont(this);
     }
   }
 
-#endif
+  // draw the last part
+  if (aLength-start)
+    DrawString(aString+start, aLength-start, aX, aY, fontPS, 
+               aSpacing?aSpacing+start:nsnull);
 
   return NS_OK;
 }
@@ -1121,9 +1144,10 @@ nsRenderingContextPS :: DrawString(const PRUnichar *aString, PRUint32 aLength,
                                     nscoord aX, nscoord aY, PRInt32 aFontID,
                                     const nscoord* aSpacing)
 {
-PRInt32         x = aX;
-PRInt32         y = aY;
-nsIFontMetrics  *fMetrics;
+  NS_ENSURE_TRUE(mTranMatrix && mPSObj && mFontMetrics, NS_ERROR_NULL_POINTER);
+  
+  nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
+  NS_ENSURE_TRUE(metrics, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIAtom> langGroup = nsnull;
   mFontMetrics->GetLangGroup(getter_AddRefs(langGroup));
@@ -1132,39 +1156,95 @@ nsIFontMetrics  *fMetrics;
   /* build up conversion table */
   mPSObj->preshow(aString, aLength);
 
-  SetupFontAndColor();
+  if (aLength == 0)
+    return NS_OK;
+  nsFontPS* fontPS = nsFontPS::FindFont(aString[0], *metrics->GetFont(), metrics);
+  NS_ENSURE_TRUE(fontPS, NS_ERROR_FAILURE);
+  fontPS->SetupFont(this);
 
-  if (nsnull != aSpacing){
+  PRUint32 i, start = 0;
+  for (i=0; i<aLength; i++) {
+    nsFontPS* fontThisChar;
+    fontThisChar = nsFontPS::FindFont(aString[i], *metrics->GetFont(), metrics);
+    NS_ENSURE_TRUE(fontThisChar, NS_ERROR_FAILURE);
+    if (fontThisChar != fontPS) {
+      // draw text up to this point
+      aX += DrawString(aString+start, i-start, aX, aY, fontPS, 
+                       aSpacing?aSpacing+start:nsnull);
+      start = i;
+
+      // setup for following text
+      fontPS = fontThisChar;
+      fontPS->SetupFont(this);
+    }
+  }
+
+  // draw the last part
+  if (aLength-start)
+    DrawString(aString+start, aLength-start, aX, aY, fontPS, 
+               aSpacing?aSpacing+start:nsnull);
+
+  return NS_OK;
+}
+
+PRInt32 
+nsRenderingContextPS::DrawString(const char *aString, PRUint32 aLength,
+                                 nscoord &aX, nscoord &aY, nsFontPS* aFontPS,
+                                 const nscoord* aSpacing)
+{
+  nscoord width = 0;
+  PRInt32 x = aX;
+  PRInt32 y = aY;
+
+  PRInt32 dxMem[500];
+  PRInt32* dx0 = 0;
+  if (aSpacing) {
+    dx0 = dxMem;
+    if (aLength > 500) {
+      dx0 = new PRInt32[aLength];
+      NS_ENSURE_TRUE(dx0, NS_ERROR_OUT_OF_MEMORY);
+    }
+    mTranMatrix->ScaleXCoords(aSpacing, aLength, dx0);
+  }
+
+  mTranMatrix->TransformCoord(&x, &y);
+  width = aFontPS->DrawString(this, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aString, aLength);
+
+  if ((aSpacing) && (dx0 != dxMem)) {
+    delete [] dx0;
+  }
+
+  return width;
+}
+
+
+PRInt32 
+nsRenderingContextPS::DrawString(const PRUnichar *aString, PRUint32 aLength,
+                                 nscoord aX, nscoord aY, nsFontPS* aFontPS,
+                                 const nscoord* aSpacing)
+{
+  nscoord width = 0;
+  PRInt32 x = aX;
+  PRInt32 y = aY;
+
+  if (aSpacing) {
     // Slow, but accurate rendering
     const PRUnichar* end = aString + aLength;
     while (aString < end){
       x = aX;
       y = aY;
       mTranMatrix->TransformCoord(&x, &y);
-	    PostscriptTextOut((PRUnichar *)aString, 1, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aFontID, aSpacing, PR_TRUE);
+      aFontPS->DrawString(this, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aString, 1);
       aX += *aSpacing++;
       aString++;
     }
+    width = aX;
   } else {
     mTranMatrix->TransformCoord(&x, &y);
-	  PostscriptTextOut(aString, aLength, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aFontID, aSpacing, PR_TRUE);
+    width = aFontPS->DrawString(this, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aString, aLength);
   }
 
-  fMetrics = mFontMetrics;
-
-  if (fMetrics) {
-    const nsFont *font;
-    fMetrics->GetFont(font);
-    PRUint8 decorations = font->decorations;
-
-    if (decorations & NS_FONT_DECORATION_OVERLINE){
-      nscoord offset;
-      nscoord size;
-      fMetrics->GetUnderline(offset, size);
-      //FillRect(aX, aY, aWidth, size);
-    }
-  }
-  return NS_OK;
+  return width;
 }
 
 /** ---------------------------------------------------
@@ -1178,100 +1258,6 @@ nsRenderingContextPS :: DrawString(const nsString& aString,nscoord aX, nscoord a
   return DrawString(aString.get(), aString.Length(), aX, aY, aFontID, aSpacing);
 }
 
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS :: DrawImage(nsIImage *aImage, nscoord aX, nscoord aY)
-{
-nscoord width, height;
-
-  width = NSToCoordRound(mP2T * aImage->GetWidth());
-  height = NSToCoordRound(mP2T * aImage->GetHeight());
-
-  return DrawImage(aImage, aX, aY, width, height);
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS :: DrawImage(nsIImage *aImage, nscoord aX, nscoord aY,
-                                        nscoord aWidth, nscoord aHeight) 
-{
-nsRect  tr;
-
-  tr.x = aX;
-  tr.y = aY;
-  tr.width = aWidth;
-  tr.height = aHeight;
-
-  return DrawImage(aImage, tr);
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS :: DrawImage(nsIImage *aImage, const nsRect& aSRect, const nsRect& aDRect)
-{
-nsRect	sr,dr;
-
-	sr = aSRect;
-	mTranMatrix->TransformCoord(&sr.x, &sr.y, &sr.width, &sr.height);
-  sr.x = aSRect.x;
-  sr.y = aSRect.y;
-  mTranMatrix->TransformNoXLateCoord(&sr.x, &sr.y);
-
-  dr = aDRect;
-	mTranMatrix->TransformCoord(&dr.x, &dr.y, &dr.width, &dr.height);
-
-
-  mPSObj->colorimage(aImage,NS_PIXELS_TO_POINTS(sr.x),
-              NS_PIXELS_TO_POINTS(sr.y),
-              NS_PIXELS_TO_POINTS(dr.width),
-              NS_PIXELS_TO_POINTS(dr.height));
-  //return aImage->Draw(*this, mSurface, 0, 0, sr.width, sr.height, dr.x, dr.y, dr.width, dr.height);
-
-  return NS_OK;
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS :: DrawImage(nsIImage *aImage, const nsRect& aRect)
-{
-nsRect	tr;
-
-	tr = aRect;
-	mTranMatrix->TransformCoord(&tr.x, &tr.y, &tr.width, &tr.height);
-
-  //return aImage->Draw(*this, mSurface, tr.x, tr.y, tr.width, tr.height);
-  mPSObj->colorimage(aImage,
-                        NS_PIXELS_TO_POINTS(tr.x),
-                        NS_PIXELS_TO_POINTS(tr.y), 
-                        NS_PIXELS_TO_POINTS(tr.width),
-                        NS_PIXELS_TO_POINTS(tr.height));
-  return NS_OK;
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 3/16/00 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS::DrawTile(nsIImage *aImage,nscoord aX0,nscoord aY0,nscoord aX1,nscoord aY1,
-                                                    nscoord aWidth,nscoord aHeight)
-{
-  return NS_OK;
-}
-
-
 /* [noscript] void drawImage (in imgIContainer aImage, [const] in nsRect aSrcRect, [const] in nsPoint aDestPoint); */
 NS_IMETHODIMP nsRenderingContextPS::DrawImage(imgIContainer *aImage, const nsRect * aSrcRect, const nsPoint * aDestPoint)
 {
@@ -1390,35 +1376,6 @@ NS_IMETHODIMP nsRenderingContextPS::RetrieveCurrentNativeGraphicData(PRUint32 *
   return NS_OK;
 }
 
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-void 
-nsRenderingContextPS :: SetupFontAndColor(void)
-{
-nscoord         fontHeight = 0;
-PRInt16         fontIndex;
-const nsFont    *font;
-nsFontHandle    fontHandle; 
-nsAutoString    fontFamily;
-
-  mFontMetrics->GetHeight(fontHeight);
-  mFontMetrics->GetFont(font);
-  mFontMetrics->GetFontHandle(fontHandle);
-
-  //mStates->mFont = mCurrFont = tfont;
-  mStates->mFontMetrics = mFontMetrics;
-
-  nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
-
-  // get the fontfamily we are using, not what we want, but what we are using
-  fontFamily.AssignWithConversion(metrics->mAFMInfo->mPSFontInfo->mFamilyName);
-  fontIndex = metrics->GetFontIndex();
-
-  mPSObj->setscriptfont(fontIndex,fontFamily,fontHeight,font->style,font->variant,font->weight,font->decorations);
-}
-
 /** ---------------------------------------------------
  *  See documentation in nsRenderingContextPS.h
  *	@update 3/24/2000 yueheng.xu@intel.com
@@ -1458,7 +1415,7 @@ const nsFont    *font;
 
   mPSObj->moveto(aX, aY);
   if (PR_TRUE == aIsUnicode) {  
-    mPSObj->show(aString, aLength, "");
+    mPSObj->show(aString, aLength, "", 0);
   }
 }
 
