@@ -188,7 +188,7 @@ JS_DHashTableInit(JSDHashTable *table, JSDHashTableOps *ops, void *data,
         capacity = JS_DHASH_MIN_SIZE;
     log2 = JS_CeilingLog2(capacity);
     capacity = JS_BIT(log2);
-    if (capacity >= JS_DHASH_MAX_SIZE)
+    if (capacity >= JS_DHASH_SIZE_LIMIT)
         return JS_FALSE;
     table->hashShift = JS_DHASH_BITS - log2;
     table->maxAlphaFrac = 0xC0;                 /* 12/16 or .75 */
@@ -307,7 +307,7 @@ JS_DHashTableFinish(JSDHashTable *table)
     }
 #endif
 
-    /* Call finalize before clearing entries. */
+    /* Call finalize before clearing entries, so it can enumerate them. */
     table->ops->finalize(table);
 
     /* Clear any remaining live entries. */
@@ -428,7 +428,7 @@ ChangeTable(JSDHashTable *table, int deltaLog2)
     newLog2 = oldLog2 + deltaLog2;
     oldCapacity = JS_BIT(oldLog2);
     newCapacity = JS_BIT(newLog2);
-    if (newCapacity >= JS_DHASH_MAX_SIZE)
+    if (newCapacity >= JS_DHASH_SIZE_LIMIT)
         return JS_FALSE;
     entrySize = table->entrySize;
     nbytes = newCapacity * entrySize;
@@ -475,10 +475,11 @@ JS_DHashTableOperate(JSDHashTable *table, const void *key, JSDHashOperator op)
     uint32 size;
     int deltaLog2;
 
-    /* Avoid 0 and 1 hash codes, they indicate free and removed entries. */
     keyHash = table->ops->hashKey(table, key);
-    ENSURE_LIVE_KEYHASH(keyHash);
     keyHash *= JS_DHASH_GOLDEN_RATIO;
+
+    /* Avoid 0 and 1 hash codes, they indicate free and removed entries. */
+    ENSURE_LIVE_KEYHASH(keyHash);
     keyHash &= ~COLLISION_FLAG;
 
     switch (op) {
@@ -568,6 +569,7 @@ JS_DHashTableRawRemove(JSDHashTable *table, JSDHashEntryHdr *entry)
 {
     JSDHashNumber keyHash;      /* load first in case clearEntry goofs it */
 
+    JS_ASSERT(JS_DHASH_ENTRY_IS_LIVE(entry));
     keyHash = entry->keyHash;
     table->ops->clearEntry(table, entry);
     if (keyHash & COLLISION_FLAG) {
@@ -585,6 +587,7 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
 {
     char *entryAddr, *entryLimit;
     uint32 i, capacity, entrySize;
+    JSBool didRemove;
     JSDHashEntryHdr *entry;
     JSDHashOperator op;
 
@@ -593,6 +596,7 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
     capacity = JS_DHASH_TABLE_SIZE(table);
     entryLimit = entryAddr + capacity * entrySize;
     i = 0;
+    didRemove = JS_FALSE;
     while (entryAddr < entryLimit) {
         entry = (JSDHashEntryHdr *)entryAddr;
         if (ENTRY_IS_LIVE(entry)) {
@@ -600,6 +604,7 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
             if (op & JS_DHASH_REMOVE) {
                 METER(table->stats.removeEnums++);
                 JS_DHashTableRawRemove(table, entry);
+                didRemove = JS_TRUE;
             }
             if (op & JS_DHASH_STOP)
                 break;
@@ -610,11 +615,14 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
     /*
      * Shrink or compress if a quarter or more of all entries are removed, or
      * if the table is underloaded according to the configured minimum alpha,
-     * and is not minimal-size already.
+     * and is not minimal-size already.  Do this only if we removed above, so
+     * non-removing enumerations can count on stable table->entryStore until
+     * the next non-lookup-Operate or removing-Enumerate.
      */
-    if (table->removedCount >= capacity >> 2 ||
-        (capacity > JS_DHASH_MIN_SIZE &&
-         table->entryCount <= MIN_LOAD(table, capacity))) {
+    if (didRemove &&
+        (table->removedCount >= capacity >> 2 ||
+         (capacity > JS_DHASH_MIN_SIZE &&
+          table->entryCount <= MIN_LOAD(table, capacity)))) {
         METER(table->stats.enumShrinks++);
         capacity = table->entryCount;
         capacity += capacity >> 1;
