@@ -21,10 +21,6 @@
  *   Robert Ginda <rginda@netscape.com>
  */
 
-#ifdef XPCONNECT_STANDALONE
-#define NO_SUBSCRIPT_LOADER
-#endif
-
 #include "prlog.h"
 
 #include "nsCOMPtr.h"
@@ -42,7 +38,6 @@
 #include "nsIXPConnect.h"
 #include "nsCRT.h"
 #include "nsMemory.h"
-#include "nsIRegistry.h"
 #include "nsXPIDLString.h"
 #include "nsIObserverService.h"
 #include "nsIXPCScriptable.h"
@@ -53,10 +48,7 @@
 #include "nsIStandardURL.h"
 #include "nsNetUtil.h"
 #endif
-#ifndef NO_SUBSCRIPT_LOADER
-#include "mozJSSubScriptLoader.h"
-#endif
-
+#include "nsIComponentLoaderManager.h"
 // For reporting errors with the console service
 #include "nsIScriptError.h"
 #include "nsIConsoleService.h"
@@ -64,10 +56,6 @@
 const char mozJSComponentLoaderContractID[] = "@mozilla.org/moz/jsloader;1";
 const char jsComponentTypeName[] = "text/javascript";
 
-#ifndef NO_SUBSCRIPT_LOADER
-const char mozJSSubScriptLoadContractID[] = "@mozilla.org/moz/jssubscript-loader;1";
-#endif
-
 // same as in nsComponentManager.cpp (but without the JS)
 const char JSfileSizeValueName[] = "FileSize";
 const char JSlastModValueName[] = "LastModTimeStamp";
@@ -329,7 +317,6 @@ public:
   BackstagePass(nsIPrincipal *prin) :
     mPrincipal(prin)
   {
-    NS_INIT_ISUPPORTS(); 
   }
 
   virtual ~BackstagePass() { }
@@ -350,7 +337,6 @@ public:
 
   BackstagePass()
   {
-    NS_INIT_ISUPPORTS(); 
   }
 
   virtual ~BackstagePass() { }
@@ -391,14 +377,11 @@ BackstagePass::NewResolve(nsIXPConnectWrappedNative *wrapper,
 }
 
 mozJSComponentLoader::mozJSComponentLoader()
-    : mCompMgr(nsnull),
-      mRuntime(nsnull),
+    : mRuntime(nsnull),
       mModules(nsnull),
       mGlobals(nsnull),
-      mXPCOMKey(0),
       mInitialized(PR_FALSE)
 {
-    NS_INIT_REFCNT();
 }
 
 static PRIntn PR_CALLBACK
@@ -429,7 +412,8 @@ mozJSComponentLoader::~mozJSComponentLoader()
 {
     if (mInitialized) {
         mInitialized = PR_FALSE;
-        PL_HashTableEnumerateEntries(mModules, UnloadAndReleaseModules,
+        PL_HashTableEnumerateEntries(mModules, 
+                                     UnloadAndReleaseModules,
                                      mCompMgr);
         PL_HashTableDestroy(mModules);
         mModules = nsnull;
@@ -480,17 +464,12 @@ NS_IMETHODIMP
 mozJSComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
 {
     mCompMgr = aCompMgr;
+         
     nsresult rv;
+    mLoaderManager = do_QueryInterface(mCompMgr, &rv);
+    if (NS_FAILED(rv))
+        return rv;
 
-    /* initialize registry handles */
-    mRegistry = do_QueryInterface(aReg, &rv);
-    if (NS_SUCCEEDED(rv)) {
-        rv = mRegistry->GetSubtree(nsIRegistry::Common, JSxpcomKeyName,
-                                   &mXPCOMKey);
-        if (NS_FAILED(rv))
-            /* if we can't get the XPCOM key, just skip all registry ops */
-            mRegistry = nsnull;
-    }
     return NS_OK;
 }
 
@@ -605,75 +584,28 @@ mozJSComponentLoader::SetRegistryInfo(const char *registryLocation,
                                       nsIFile *component)
 {
     nsresult rv;
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-    eRegistryLocation = (char*)registryLocation;
-
-
-    nsRegistryKey key;
-
-    rv = mRegistry->AddSubtreeRaw(mXPCOMKey, eRegistryLocation, &key);
-    if (registryLocation != eRegistryLocation)
-        nsMemory::Free(eRegistryLocation);
-
-    if (NS_FAILED(rv))
-        return rv;
-
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
+    
     PRInt64 modDate;
-
-    if (NS_FAILED(rv = component->GetLastModifiedTime(&modDate)) ||
-        NS_FAILED(rv = mRegistry->SetLongLong(key, JSlastModValueName, &modDate)))
-        return rv;
-
-    PRInt64 fileSize;
-    if (NS_FAILED(rv = component->GetFileSize(&fileSize)) ||
-        NS_FAILED(rv = mRegistry->SetLongLong(key, JSfileSizeValueName, &fileSize)))
+    rv = component->GetLastModifiedTime(&modDate);
+    if (NS_FAILED(rv))
         return rv;
 
 #ifdef DEBUG_shaver_off
     fprintf(stderr, "SetRegistryInfo(%s) => (%d,%d)\n", registryLocation,
             modDate, fileSize);
 #endif
-
-    return NS_OK;
+    return mLoaderManager->SaveFileInfo(component, registryLocation, modDate);                                                          
 }
 
 nsresult
-mozJSComponentLoader::RemoveRegistryInfo(const char *registryLocation)
+mozJSComponentLoader::RemoveRegistryInfo(nsIFile *component, const char *registryLocation)
 {
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
-    nsresult rv;
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-    eRegistryLocation = (char*)registryLocation;
-
-
-    rv = mRegistry->RemoveSubtree(mXPCOMKey, eRegistryLocation);
-        
-    if (registryLocation != eRegistryLocation)
-            nsMemory::Free(eRegistryLocation);
-
-    return rv;
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
+    
+    return mLoaderManager->RemoveFileInfo(component, registryLocation);                                                          
 }
 
 
@@ -681,46 +613,15 @@ PRBool
 mozJSComponentLoader::HasChanged(const char *registryLocation,
                                  nsIFile *component)
 {
-
-    /* if we don't have a registry handle, force registration of component */
-    if (!mRegistry)
-        return PR_TRUE;
-
-    nsresult rv;
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-        eRegistryLocation = (char*)registryLocation;
-
-    nsRegistryKey key;
-    int r = NS_FAILED(mRegistry->GetSubtreeRaw(mXPCOMKey, eRegistryLocation, &key));
-    if (registryLocation != eRegistryLocation)
-        nsMemory::Free(eRegistryLocation);
-    if (r)
-        return PR_TRUE;
-
-    /* check modification date */
-    PRInt64 regTime, lastTime;
-    if (NS_FAILED(mRegistry->GetLongLong(key, JSlastModValueName, &regTime)))
-        return PR_TRUE;
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
     
-    if (NS_FAILED(component->GetLastModifiedTime(&lastTime)) || LL_NE(lastTime, regTime))
-        return PR_TRUE;
+    PRInt64 lastTime;
+    component->GetLastModifiedTime(&lastTime);
 
-    /* check file size */
-    PRInt64 regSize;
-    if (NS_FAILED(mRegistry->GetLongLong(key, JSfileSizeValueName, &regSize)))
-        return PR_TRUE;
-    PRInt64 size;
-    if (NS_FAILED(component->GetFileSize(&size)) || LL_NE(size,regSize) )
-        return PR_TRUE;
-
-    return PR_FALSE;
+    PRBool hasChanged = PR_TRUE;
+    mLoaderManager->HasFileChanged(component, registryLocation, lastTime, &hasChanged);                                                          
+    return hasChanged;
 }
 
 NS_IMETHODIMP
@@ -829,7 +730,7 @@ mozJSComponentLoader::AttemptRegistration(nsIFile *component,
     
     /* no need to check registry data on deferred reg */
     if (!deferred && !HasChanged(registryLocation, component))
-        goto out;
+        return NS_OK;
     
     module = ModuleForLocation(registryLocation, component);
     if (!module)
@@ -926,7 +827,7 @@ mozJSComponentLoader::UnregisterComponent(nsIFile *component)
     if (NS_SUCCEEDED(rv))
     {
         // Remove any autoreg specific info. Ignore error.
-        RemoveRegistryInfo(registryLocation);
+        RemoveRegistryInfo(component, registryLocation);
     }
         
     return rv;
@@ -1315,57 +1216,3 @@ JSCLAutoContext::~JSCLAutoContext()
 //----------------------------------------------------------------------
 
 /* XXX this should all be data-driven, via NS_IMPL_GETMODULE_WITH_CATEGORIES */
-static NS_METHOD
-RegisterJSLoader(nsIComponentManager *aCompMgr, nsIFile *aPath,
-                 const char *registryLocation, const char *componentType,
-                 const nsModuleComponentInfo *info)
-{
-    nsresult rv;
-    nsCOMPtr<nsICategoryManager> catman =
-        do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    nsXPIDLCString previous;
-    return catman->AddCategoryEntry("component-loader", jsComponentTypeName,
-                                    mozJSComponentLoaderContractID,
-                                    PR_TRUE, PR_TRUE, getter_Copies(previous));
-}
-
-static NS_METHOD
-UnregisterJSLoader(nsIComponentManager *aCompMgr, nsIFile *aPath,
-                   const char *registryLocation,
-                   const nsModuleComponentInfo *info)
-{
-    nsresult rv;
-    nsCOMPtr<nsICategoryManager> catman =
-        do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    nsXPIDLCString jsLoader;
-    rv = catman->GetCategoryEntry("component-loader", jsComponentTypeName,
-                                  getter_Copies(jsLoader));
-    if (NS_FAILED(rv)) return rv;
-
-    // only unregister if we're the current JS component loader
-    if (!strcmp(jsLoader, mozJSComponentLoaderContractID)) {
-        return catman->DeleteCategoryEntry("component-loader",
-                                           jsComponentTypeName, PR_TRUE);
-    }
-    return NS_OK;
-}
-
-NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSComponentLoader);
-
-#ifndef NO_SUBSCRIPT_LOADER
-NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSSubScriptLoader);
-#endif
-
-static const nsModuleComponentInfo components[] = {
-    { "JS component loader", MOZJSCOMPONENTLOADER_CID,
-      mozJSComponentLoaderContractID, mozJSComponentLoaderConstructor,
-      RegisterJSLoader, UnregisterJSLoader },
-#ifndef NO_SUBSCRIPT_LOADER
-   { "JS subscript loader", MOZ_JSSUBSCRIPTLOADER_CID,
-      mozJSSubScriptLoadContractID, mozJSSubScriptLoaderConstructor }
-#endif
-};
-
-NS_IMPL_NSGETMODULE(JS_component_loader, components);
