@@ -50,6 +50,7 @@
 #include "nsIEventStateManager.h"
 #include "nsIObserver.h"
 #include "nsILookAndFeel.h"
+#include "nsIIOService.h"
 #ifdef IBMBIDI
 #include "nsBidiUtils.h"
 #endif
@@ -57,6 +58,7 @@
 #include "nsHashtable.h"
 #include "nsIContent.h"
 #include "nsITheme.h"
+#include "nsWeakReference.h"
 
 // Base class for concrete presentation context classes
 class nsPresContext : public nsIPresContext, public nsIObserver {
@@ -78,6 +80,7 @@ public:
   NS_IMETHOD SetImageAnimationMode(PRUint16 aMode);
   NS_IMETHOD GetImageLoadFlags(nsLoadFlags& aLoadFlags);
   NS_IMETHOD GetLookAndFeel(nsILookAndFeel** aLookAndFeel);
+  NS_IMETHOD GetIOService(nsIIOService** aIOService);
   NS_IMETHOD GetBaseURL(nsIURI** aURLResult);
   NS_IMETHOD GetMedium(nsIAtom** aMediumResult) = 0;
   NS_IMETHOD ClearStyleDataAndReflow(void);
@@ -99,6 +102,7 @@ public:
                                         nsIAtom* aPseudoTag,
                                         nsIStyleContext* aParentContext,
                                         nsIStyleContext** aResult);
+  NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult);
   NS_IMETHOD ReParentStyleContext(nsIFrame* aFrame, 
                                   nsIStyleContext* aNewParentContext);
   NS_IMETHOD GetMetricsFor(const nsFont& aFont, nsIFontMetrics** aResult);
@@ -113,10 +117,6 @@ public:
   NS_IMETHOD SetFontScaler(PRInt32 aScaler);
   NS_IMETHOD GetDefaultColor(nscolor* aColor);
   NS_IMETHOD GetDefaultBackgroundColor(nscolor* aColor);
-  NS_IMETHOD GetDefaultBackgroundImage(nsString& aImage);
-  NS_IMETHOD GetDefaultBackgroundImageRepeat(PRUint8* aRepeat);
-  NS_IMETHOD GetDefaultBackgroundImageOffset(nscoord* aX, nscoord* aY);
-  NS_IMETHOD GetDefaultBackgroundImageAttachment(PRUint8* aRepeat);
   NS_IMETHOD GetDefaultLinkColor(nscolor* aColor);
   NS_IMETHOD GetDefaultVisitedLinkColor(nscolor* aColor);
 
@@ -127,10 +127,6 @@ public:
   NS_IMETHOD GetFocusRingOnAnything(PRBool& focusRingOnAnything);
   NS_IMETHOD SetDefaultColor(nscolor aColor);
   NS_IMETHOD SetDefaultBackgroundColor(nscolor aColor);
-  NS_IMETHOD SetDefaultBackgroundImage(const nsString& aImage);
-  NS_IMETHOD SetDefaultBackgroundImageRepeat(PRUint8 aRepeat);
-  NS_IMETHOD SetDefaultBackgroundImageOffset(nscoord aX, nscoord aY);
-  NS_IMETHOD SetDefaultBackgroundImageAttachment(PRUint8 aRepeat);
   NS_IMETHOD SetDefaultLinkColor(nscolor aColor);
   NS_IMETHOD SetDefaultVisitedLinkColor(nscolor aColor);
 
@@ -185,20 +181,25 @@ public:
   NS_IMETHOD SetVisualMode(PRBool aIsVisual);
   NS_IMETHOD GetBidiUtils(nsBidiPresUtils** aBidiUtils);
   NS_IMETHOD SetBidi(PRUint32 aSource, PRBool aForceReflow = PR_FALSE);
-  NS_IMETHOD GetBidi(PRUint32* aDest);
+  NS_IMETHOD GetBidi(PRUint32* aDest) const;
  //ahmed
-  NS_IMETHOD IsVisRTL(PRBool &aResult);
-  NS_IMETHOD IsArabicEncoding(PRBool &aResult);
+  NS_IMETHOD IsVisRTL(PRBool &aResult) const;
+  NS_IMETHOD IsArabicEncoding(PRBool &aResult) const;
 
 //Mohamed  17-1-01
   NS_IMETHOD SetIsBidiSystem(PRBool aIsBidi);
   NS_IMETHOD GetIsBidiSystem(PRBool &aResult) const;
-  NS_IMETHOD GetBidiCharset(nsAString &aCharSet);
+  NS_IMETHOD GetBidiCharset(nsAString &aCharSet) const;
 //Mohamed End
 #endif // IBMBIDI
 
   NS_IMETHOD GetTheme(nsITheme** aResult);
   NS_IMETHOD ThemeChanged();
+  NS_IMETHOD SysColorChanged();
+  NS_IMETHOD FindFrameBackground(nsIFrame* aFrame,
+                                 const nsStyleBackground** aBackground,
+                                 PRBool* aIsCanvas,
+                                 PRBool* aFoundBackground);
 
 protected:
   nsPresContext();
@@ -218,8 +219,9 @@ protected:
   nsCOMPtr<nsILanguageAtom> mLanguage;
   nsLanguageSpecificTransformType mLanguageSpecificTransformType;
   nsILinkHandler*       mLinkHandler;   // [WEAK]
-  nsISupports*          mContainer;     // [WEAK]
+  nsWeakPtr             mContainer;
   nsCOMPtr<nsILookAndFeel> mLookAndFeel;
+  nsCOMPtr<nsIIOService> mIOService;
 
   nsFont                mDefaultVariableFont;
   nsFont                mDefaultFixedFont;
@@ -247,14 +249,8 @@ protected:
   PRUint8               mFocusRingWidth;          // set in GetUserPrefs
   PRPackedBool          mFocusRingOnAnything;     // set in GetUserPrefs
 
-  PRPackedBool  				mDrawImageBackground;
-  PRPackedBool  				mDrawColorBackground;
-  PRUint8               mDefaultBackgroundImageAttachment;
-  PRUint8               mDefaultBackgroundImageRepeat;
-
-  nscoord               mDefaultBackgroundImageOffsetX;
-  nscoord               mDefaultBackgroundImageOffsetY;
-  nsString              mDefaultBackgroundImage;
+  PRPackedBool          mDrawImageBackground;
+  PRPackedBool          mDrawColorBackground;
 
   nsSupportsHashtable   mImageLoaders;
 
@@ -262,7 +258,6 @@ protected:
   nsCOMPtr<nsIURI>      mBaseURL;
 
   nsCompatibility       mCompatibilityMode;
-  PRPackedBool          mCompatibilityLocked;
   nsWidgetRendering     mWidgetRenderingMode;
   PRPackedBool          mImageAnimationStopped;   // image animation stopped
 
