@@ -36,6 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 #include "nsCOMPtr.h"
 #include "nsInlineFrame.h"
+#include "nsBlockFrame.h"
 #include "nsHTMLAtoms.h"
 #include "nsHTMLParts.h"
 #include "nsIStyleContext.h"
@@ -45,7 +46,12 @@
 #include "nsIFontMetrics.h"
 #include "nsAbsoluteContainingBlock.h"
 #include "nsLayoutAtoms.h"
+#include "nsCSSAnonBoxes.h"
 #include "nsReflowPath.h"
+#ifdef ACCESSIBILITY
+#include "nsIServiceManager.h"
+#include "nsIAccessibilityService.h"
+#endif
 
 #ifdef DEBUG
 #undef NOISY_PUSHING
@@ -138,10 +144,14 @@ NS_IMETHODIMP
 nsInlineFrame::IsEmpty(nsCompatibility aCompatMode, PRBool aIsPre,
                        PRBool* aResult)
 {
+#if 0
+  // I used to think inline frames worked this way, but it seems they
+  // don't.  At least not in our codebase.
   if (aCompatMode == eCompatibility_FullStandards) {
     *aResult = PR_FALSE;
     return NS_OK;
   }
+#endif
   const nsStyleMargin* margin = NS_STATIC_CAST(const nsStyleMargin*,
                              mStyleContext->GetStyleData(eStyleStruct_Margin));
   const nsStyleBorder* border = NS_STATIC_CAST(const nsStyleBorder*,
@@ -149,32 +159,21 @@ nsInlineFrame::IsEmpty(nsCompatibility aCompatMode, PRBool aIsPre,
   const nsStylePadding* padding = NS_STATIC_CAST(const nsStylePadding*,
                             mStyleContext->GetStyleData(eStyleStruct_Padding));
   nsStyleCoord coord;
-  if ((border->IsBorderSideVisible(NS_SIDE_TOP) &&
-       !IsBorderZero(border->mBorder.GetTopUnit(),
-                     border->mBorder.GetTop(coord))) ||
-      (border->IsBorderSideVisible(NS_SIDE_RIGHT) &&
-       !IsBorderZero(border->mBorder.GetTopUnit(),
-                     border->mBorder.GetTop(coord))) ||
-      (border->IsBorderSideVisible(NS_SIDE_BOTTOM) &&
-       !IsBorderZero(border->mBorder.GetTopUnit(),
-                     border->mBorder.GetTop(coord))) ||
+  // XXX Top and bottom removed, since they shouldn't affect things, but this
+  // doesn't really match with nsLineLayout.cpp's setting of
+  // ZeroEffectiveSpanBox, anymore, so what should this really be?
+  if ((border->IsBorderSideVisible(NS_SIDE_RIGHT) &&
+       !IsBorderZero(border->mBorder.GetRightUnit(),
+                     border->mBorder.GetRight(coord))) ||
       (border->IsBorderSideVisible(NS_SIDE_LEFT) &&
-       !IsBorderZero(border->mBorder.GetTopUnit(),
-                     border->mBorder.GetTop(coord))) ||
-      !IsPaddingZero(padding->mPadding.GetTopUnit(),
-                    padding->mPadding.GetTop(coord)) ||
+       !IsBorderZero(border->mBorder.GetLeftUnit(),
+                     border->mBorder.GetLeft(coord))) ||
       !IsPaddingZero(padding->mPadding.GetRightUnit(),
                     padding->mPadding.GetRight(coord)) ||
-      !IsPaddingZero(padding->mPadding.GetBottomUnit(),
-                    padding->mPadding.GetBottom(coord)) ||
       !IsPaddingZero(padding->mPadding.GetLeftUnit(),
                     padding->mPadding.GetLeft(coord)) ||
-      !IsMarginZero(margin->mMargin.GetTopUnit(),
-                    margin->mMargin.GetTop(coord)) ||
       !IsMarginZero(margin->mMargin.GetRightUnit(),
                     margin->mMargin.GetRight(coord)) ||
-      !IsMarginZero(margin->mMargin.GetBottomUnit(),
-                    margin->mMargin.GetBottom(coord)) ||
       !IsMarginZero(margin->mMargin.GetLeftUnit(),
                     margin->mMargin.GetLeft(coord))) {
     *aResult = PR_FALSE;
@@ -283,11 +282,6 @@ nsInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
       // command.
       nsIFrame* oldFrameNextInFlow;
       aOldFrame->GetNextInFlow(&oldFrameNextInFlow);
-      nsSplittableType st;
-      aOldFrame->IsSplittable(st);
-      if (NS_FRAME_NOT_SPLITTABLE != st) {
-        nsSplittableFrame::RemoveFromFlow(aOldFrame);
-      }
       parent->mFrames.DestroyFrame(aPresContext, aOldFrame);
       aOldFrame = oldFrameNextInFlow;
       if (nsnull != aOldFrame) {
@@ -311,24 +305,56 @@ nsInlineFrame::ReplaceFrame(nsIPresContext* aPresContext,
                             nsIFrame* aOldFrame,
                             nsIFrame* aNewFrame)
 {
-  if (nsnull != aListName) {
+  if (aListName) {
+    NS_ERROR("Don't have any special lists on inline frames!");
     return NS_ERROR_INVALID_ARG;
   }
   if (!aOldFrame || !aNewFrame) {
+    NS_ERROR("Missing aOldFrame or aNewFrame");
     return NS_ERROR_INVALID_ARG;
   }
 
-  // Replace the old frame with the new frame in the list, then remove the old frame
-  mFrames.ReplaceFrame(this, aOldFrame, aNewFrame);
-  aOldFrame->Destroy(aPresContext);
-
+  PRBool retval =
+    mFrames.ReplaceFrame(aPresContext, this, aOldFrame, aNewFrame, PR_TRUE);
+  
   // Ask the parent frame to reflow me.
   ReflowDirtyChild(&aPresShell, nsnull);
 
-  return NS_OK;
+  return retval ? NS_OK : NS_ERROR_FAILURE;
 }
 
 
+NS_IMETHODIMP
+nsInlineFrame::Paint(nsIPresContext*      aPresContext,
+                     nsIRenderingContext& aRenderingContext,
+                     const nsRect&        aDirtyRect,
+                     nsFramePaintLayer    aWhichLayer,
+                     PRUint32             aFlags)
+{
+  if (NS_FRAME_IS_UNFLOWABLE & mState) {
+    return NS_OK;
+  }
+
+  // Paint inline element backgrounds in the foreground layer (bug 36710).
+  if (aWhichLayer == NS_FRAME_PAINT_LAYER_FOREGROUND) {
+    PaintSelf(aPresContext, aRenderingContext, aDirtyRect);
+  }
+    
+  // The sole purpose of this is to trigger display of the selection
+  // window for Named Anchors, which don't have any children and
+  // normally don't have any size, but in Editor we use CSS to display
+  // an image to represent this "hidden" element.
+  if (!mFrames.FirstChild()) {
+    nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
+                   aWhichLayer, aFlags);
+  }
+
+  PaintDecorationsAndChildren(aPresContext, aRenderingContext,
+                              aDirtyRect, aWhichLayer, PR_FALSE,
+                              aFlags);
+  return NS_OK;
+}
+
 //////////////////////////////////////////////////////////////////////
 // Reflow methods
 
@@ -491,6 +517,7 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
     // Subtract off left and right border+padding from availableWidth
     availableWidth -= leftEdge;
     availableWidth -= aReflowState.mComputedBorderPadding.right;
+    availableWidth = PR_MAX(0, availableWidth);
   }
   lineLayout->BeginSpan(this, &aReflowState, leftEdge, leftEdge + availableWidth);
 
@@ -578,7 +605,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
   // that are empty we force to empty so that things like collapsed
   // whitespace in an inline element don't affect the line-height.
   nsSize size;
-  lineLayout->EndSpan(this, size, aMetrics.maxElementSize);
+  lineLayout->EndSpan(this, size,
+                    aMetrics.mComputeMEW ? &aMetrics.mMaxElementWidth : nsnull);
   if ((0 == size.height) && (0 == size.width) &&
       ((nsnull != mPrevInFlow) || (nsnull != mNextInFlow))) {
     // This is a continuation of a previous inline. Therefore make
@@ -587,9 +615,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
     aMetrics.height = 0;
     aMetrics.ascent = 0;
     aMetrics.descent = 0;
-    if (nsnull != aMetrics.maxElementSize) {
-      aMetrics.maxElementSize->width = 0;
-      aMetrics.maxElementSize->height = 0;
+    if (aMetrics.mComputeMEW) {
+      aMetrics.mMaxElementWidth = 0;
     }
   }
   else {
@@ -654,9 +681,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
   ListTag(stdout);
   printf(": metrics=%d,%d ascent=%d descent=%d\n",
          aMetrics.width, aMetrics.height, aMetrics.ascent, aMetrics.descent);
-  if (nsnull != aMetrics.maxElementSize) {
-    printf(" maxElementSize %d,%d\n", 
-      aMetrics.maxElementSize->width, aMetrics.maxElementSize->height);
+  if (aMetrics.mComputeMEW) {
+    printf(" maxElementWidth %d\n", aMetrics.mMaxElementWidth);
   }
 #endif
 
@@ -779,16 +805,24 @@ nsInlineFrame::ReflowInlineFrame(nsIPresContext* aPresContext,
     }
   }
   else if (NS_FRAME_IS_NOT_COMPLETE(aStatus)) {
-    nsIFrame* newFrame;
-    rv = CreateNextInFlow(aPresContext, this, aFrame, newFrame);
-    if (NS_FAILED(rv)) {
-      return rv;
+    nsCOMPtr<nsIAtom> frameType;
+    aFrame->GetFrameType(getter_AddRefs(frameType));
+    if (nsLayoutAtoms::placeholderFrame == frameType) {
+      nsBlockReflowState* blockRS = lineLayout->mBlockRS;
+      blockRS->mBlock->SplitPlaceholder(*aPresContext, *aFrame);
     }
-    if (!reflowingFirstLetter) {
-      nsIFrame* nextFrame;
-      aFrame->GetNextSibling(&nextFrame);
-      if (nsnull != nextFrame) {
-        PushFrames(aPresContext, nextFrame, aFrame);
+    else {
+      nsIFrame* newFrame;
+      rv = CreateNextInFlow(aPresContext, this, aFrame, newFrame);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
+      if (!reflowingFirstLetter) {
+        nsIFrame* nextFrame;
+        aFrame->GetNextSibling(&nextFrame);
+        if (nsnull != nextFrame) {
+          PushFrames(aPresContext, nextFrame, aFrame);
+        }
       }
     }
   }
@@ -859,7 +893,7 @@ nsInlineFrame::GetSkipSides() const
       skip |= 1 << NS_SIDE_LEFT;
     }
     else {
-      // If the prev-in-flow is empty, then go ahead and let our right
+      // If the prev-in-flow is empty, then go ahead and let our left
       // edge border render.
     }
   }
@@ -878,6 +912,31 @@ nsInlineFrame::GetSkipSides() const
   return skip;
 }
 
+#ifdef ACCESSIBILITY
+NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
+{
+  // Broken image accessibles are created here, because layout
+  // replaces the image or image control frame with an inline frame
+  *aAccessible = nsnull;
+  nsCOMPtr<nsIAtom> tagAtom;
+  mContent->GetTag(*getter_AddRefs(tagAtom));
+  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || tagAtom == nsHTMLAtoms::hr) {
+    // Only get accessibility service if we're going to use it
+    nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
+    if (!accService)
+      return NS_ERROR_FAILURE;
+    if (tagAtom == nsHTMLAtoms::input)  // Broken <input type=image ... />
+      return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    else if (tagAtom == nsHTMLAtoms::img)  // Create accessible for broken <img>
+      return accService->CreateHTMLImageAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    // Create accessible for <hr>
+    return accService->CreateHTMLHRAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+  }
+
+  return NS_ERROR_FAILURE;
+}
+#endif
+
 //////////////////////////////////////////////////////////////////////
 
 // nsLineFrame implementation
@@ -1039,10 +1098,8 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
         // we behave as if an anonymous (unstyled) span was the child
         // of the parent frame.
         nsIStyleContext* newSC;
-        aPresContext->ResolvePseudoStyleContextFor(mContent,
-                                                   nsHTMLAtoms::mozLineFrame,
-                                                   parentContext,
-                                                   &newSC);
+        aPresContext->ResolvePseudoStyleContextFor(nsnull,
+                      nsCSSAnonBoxes::mozLineFrame, parentContext, &newSC);
         if (newSC) {
           // Switch to the new style context.
           SetStyleContext(aPresContext, newSC);
@@ -1074,7 +1131,7 @@ NS_NewPositionedInlineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
   if (nsnull == aNewFrame) {
     return NS_ERROR_NULL_POINTER;
   }
-  nsPositionedInlineFrame* it = new (aPresShell) nsPositionedInlineFrame;
+  nsPositionedInlineFrame* it = new (aPresShell) nsPositionedInlineFrame();
   if (nsnull == it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1096,7 +1153,7 @@ nsPositionedInlineFrame::SetInitialChildList(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.SetInitialChildList(this, aPresContext, aListName, aChildList);
   } else {
     rv = nsInlineFrame::SetInitialChildList(aPresContext, aListName, aChildList);
@@ -1113,7 +1170,7 @@ nsPositionedInlineFrame::AppendFrames(nsIPresContext* aPresContext,
 {
   nsresult  rv;
   
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.AppendFrames(this, aPresContext, aPresShell, aListName,
                                          aFrameList);
   } else {
@@ -1133,7 +1190,7 @@ nsPositionedInlineFrame::InsertFrames(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.InsertFrames(this, aPresContext, aPresShell, aListName,
                                          aPrevFrame, aFrameList);
   } else {
@@ -1152,7 +1209,7 @@ nsPositionedInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.RemoveFrame(this, aPresContext, aPresShell, aListName, aOldFrame);
   } else {
     rv = nsInlineFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
@@ -1161,6 +1218,22 @@ nsPositionedInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
   return rv;
 }
 
+NS_IMETHODIMP
+nsPositionedInlineFrame::ReplaceFrame(nsIPresContext* aPresContext,
+                                      nsIPresShell&   aPresShell,
+                                      nsIAtom*        aListName,
+                                      nsIFrame*       aOldFrame,
+                                      nsIFrame*       aNewFrame)
+{
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
+    return mAbsoluteContainer.ReplaceFrame(this, aPresContext, aPresShell,
+                                           aListName, aOldFrame, aNewFrame);
+  } else {
+    return nsInlineFrame::ReplaceFrame(aPresContext, aPresShell, aListName,
+                                       aOldFrame, aNewFrame);
+  }
+}
+
 NS_IMETHODIMP
 nsPositionedInlineFrame::GetAdditionalChildListName(PRInt32   aIndex,
                                                     nsIAtom** aListName) const
@@ -1168,7 +1241,7 @@ nsPositionedInlineFrame::GetAdditionalChildListName(PRInt32   aIndex,
   NS_PRECONDITION(nsnull != aListName, "null OUT parameter pointer");
   *aListName = nsnull;
   if (0 == aIndex) {
-    *aListName = nsLayoutAtoms::absoluteList;
+    *aListName = mAbsoluteContainer.GetChildListName();
     NS_ADDREF(*aListName);
   }
   return NS_OK;
@@ -1180,7 +1253,7 @@ nsPositionedInlineFrame::FirstChild(nsIPresContext* aPresContext,
                                     nsIFrame**      aFirstChild) const
 {
   NS_PRECONDITION(nsnull != aFirstChild, "null OUT parameter pointer");
-  if (aListName == nsLayoutAtoms::absoluteList) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     return mAbsoluteContainer.FirstChild(this, aListName, aFirstChild);
   }
 
@@ -1206,19 +1279,17 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
 
   nsRect oldRect(mRect);
 
-  // See if it's an incremental reflow command and we're not the target
+  // See if it's an incremental reflow command
   if (mAbsoluteContainer.HasAbsoluteFrames() &&
-      eReflowReason_Incremental == aReflowState.reason &&
-      !aReflowState.path->mReflowCommand) {
+      eReflowReason_Incremental == aReflowState.reason) {
     // Give the absolute positioning code a chance to handle it
     PRBool  handled;
-    nsRect  childBounds;
     nscoord containingBlockWidth = -1;
     nscoord containingBlockHeight = -1;
     
     mAbsoluteContainer.IncrementalReflow(this, aPresContext, aReflowState,
                                          containingBlockWidth, containingBlockHeight,
-                                         handled, childBounds);
+                                         handled);
 
     // If the incremental reflow command was handled by the absolute positioning
     // code, then we're all done
@@ -1235,6 +1306,8 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
       // code seems to reset the NS_FRAME_OUTSIDE_CHILDREN and so it is ignored
 #if 0
       // Factor the absolutely positioned child bounds into the overflow area
+      nsRect childBounds;
+      mAbsoluteContainer.CalculateChildBounds(aPresContext, childBounds);
       aDesiredSize.mOverflowArea.UnionRect(aDesiredSize.mOverflowArea, childBounds);
 
       // Make sure the NS_FRAME_OUTSIDE_CHILDREN flag is set correctly
@@ -1274,7 +1347,7 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
 
     rv = mAbsoluteContainer.Reflow(this, aPresContext, aReflowState,
                                    containingBlockWidth, containingBlockHeight,
-                                   childBounds);
+                                   &childBounds);
     
     // XXX Although this seems like the correct thing to do the line layout
     // code seems to reset the NS_FRAME_OUTSIDE_CHILDREN and so it is ignored
