@@ -67,7 +67,8 @@ struct nsTableCellReflowState : public nsHTMLReflowState
                          const nsSize&            aAvailableSpace,
                          nsReflowReason           aReason);
 
-  void FixUp(const nsSize& aAvailSpace);
+  void FixUp(const nsSize& aAvailSpace,
+             PRBool        aResetComputedWidth);
 };
 
 nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresContext,
@@ -79,11 +80,15 @@ nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresCon
 {
 }
 
-void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace)
+void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace,
+                                   PRBool        aResetComputedWidth)
 {
   // fix the mComputed values during a pass 2 reflow since the cell can be a percentage base
   if (NS_UNCONSTRAINEDSIZE != aAvailSpace.width) {
-    if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
+    if (aResetComputedWidth) {
+      mComputedWidth = NS_UNCONSTRAINEDSIZE;
+    }
+    else if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
       mComputedWidth = aAvailSpace.width - mComputedBorderPadding.left - mComputedBorderPadding.right;
       mComputedWidth = PR_MAX(0, mComputedWidth);
     }
@@ -101,7 +106,8 @@ nsTableRowFrame::InitChildReflowState(nsIPresContext&         aPresContext,
                                       const nsSize&           aAvailSize,
                                       PRBool                  aBorderCollapse,
                                       float                   aPixelsToTwips,
-                                      nsTableCellReflowState& aReflowState)                                    
+                                      nsTableCellReflowState& aReflowState,
+                                      PRBool                  aResetComputedWidth)
 {
   nsMargin collapseBorder;
   nsMargin* pCollapseBorder = nsnull;
@@ -113,24 +119,22 @@ nsTableRowFrame::InitChildReflowState(nsIPresContext&         aPresContext,
     }
   }
   aReflowState.Init(&aPresContext, -1, -1, pCollapseBorder);
-  aReflowState.FixUp(aAvailSize);
+  aReflowState.FixUp(aAvailSize, aResetComputedWidth);
 }
 
 void 
 nsTableRowFrame::SetFixedHeight(nscoord aValue)
 {
-  if (!HasPctHeight()) {
-    nscoord height = PR_MAX(0, aValue); 
-    if (HasFixedHeight()) {
-      if (height > mStyleHeight) {
-        mStyleHeight = height;
-      }
+  nscoord height = PR_MAX(0, aValue);
+  if (HasFixedHeight()) {
+    if (height > mStyleFixedHeight) {
+      mStyleFixedHeight = height;
     }
-    else {
-      mStyleHeight = height;
-      if (height > 0) {
-        SetHasFixedHeight(PR_TRUE);
-      }
+  }
+  else {
+    mStyleFixedHeight = height;
+    if (height > 0) {
+      SetHasFixedHeight(PR_TRUE);
     }
   }
 }
@@ -141,12 +145,12 @@ nsTableRowFrame::SetPctHeight(float  aPctValue,
 {
   nscoord height = PR_MAX(0, NSToCoordRound(aPctValue * 100.0f));
   if (HasPctHeight()) {
-    if ((height > mStyleHeight) || aForce) {
-      mStyleHeight = height;
+    if ((height > mStylePctHeight) || aForce) {
+      mStylePctHeight = height;
     }
   }
   else {
-    mStyleHeight = height;
+    mStylePctHeight = height;
     if (height > 0.0f) {
       SetHasPctHeight(PR_TRUE);
     }
@@ -452,8 +456,8 @@ nsTableRowFrame::GetHeight(nscoord aPctBasis) const
   if ((aPctBasis > 0) && HasPctHeight()) {
     height = NSToCoordRound(GetPctHeight() * (float)aPctBasis);
   }
-  else if (HasFixedHeight()) {
-    height = GetFixedHeight();
+  if (HasFixedHeight()) {
+    height = PR_MAX(height, GetFixedHeight());
   }
   return PR_MAX(height, GetContentHeight());
 }
@@ -464,6 +468,7 @@ nsTableRowFrame::ResetHeight(nscoord aFixedHeight)
   SetHasFixedHeight(PR_FALSE);
   SetHasPctHeight(PR_FALSE);
   SetFixedHeight(0);
+  SetPctHeight(0);
   SetContentHeight(0);
 
   if (aFixedHeight > 0) {
@@ -943,6 +948,13 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
     if (aDirtyOnly && ((frameState & NS_FRAME_IS_DIRTY) == 0)) {
       doReflowChild = PR_FALSE;
     }
+    else if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableHeight) && IS_TABLE_CELL(frameType)) {
+      // We don't reflow a rowspan >1 cell here with a constrained height. 
+      // That happens in nsTableRowGroupFrame::SplitSpanningCells.
+      if (aTableFrame.GetEffectiveRowSpan((nsTableCellFrame&)*kidFrame) > 1) {
+        doReflowChild = PR_FALSE;
+      }
+    }
     if (aReflowState.mFlags.mSpecialHeightReflow) {
       if (!isPaginated && (IS_TABLE_CELL(frameType.get()) &&
                            !((nsTableCellFrame*)kidFrame)->NeedSpecialReflow())) {
@@ -998,7 +1010,6 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
           nsSize  kidAvailSize(availColWidth, aReflowState.availableHeight);
           nsReflowReason reason = eReflowReason_Resize;
           PRBool cellToWatch = PR_FALSE;
-          nsSize maxElementSize;
           // If it's a dirty frame, then check whether it's the initial reflow
           if (frameState & NS_FRAME_FIRST_REFLOW) {
             reason = eReflowReason_Initial;
@@ -1021,7 +1032,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
                 desiredSize.mFlags |= NS_REFLOW_CALC_MAX_WIDTH; 
               }
               // request to get the max element size 
-              desiredSize.maxElementSize = &maxElementSize;
+              desiredSize.mComputeMEW = PR_TRUE;
             }
             else {
               cellToWatch = PR_FALSE;
@@ -1044,11 +1055,11 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
             nscoord maxWidth = (NS_UNCONSTRAINEDSIZE == availCellWidth) 
                                 ? desiredSize.width : desiredSize.mMaximumWidth;
             // save the max element width and max width
-            if (desiredSize.maxElementSize) {
-              cellFrame->SetPass1MaxElementWidth(desiredSize.width, desiredSize.maxElementSize->width);
-              if (desiredSize.maxElementSize->width > desiredSize.width) {
+            if (desiredSize.mComputeMEW) {
+              cellFrame->SetPass1MaxElementWidth(desiredSize.width, desiredSize.mMaxElementWidth);
+              if (desiredSize.mMaxElementWidth > desiredSize.width) {
                 NS_ASSERTION(PR_FALSE, "max element width exceeded desired width");
-                desiredSize.width = desiredSize.maxElementSize->width;
+                desiredSize.width = desiredSize.mMaxElementWidth;
               }
             }
             cellFrame->SetMaximumWidth(maxWidth);
@@ -1275,15 +1286,20 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     nsSize  cellAvailSize(cellAvailWidth, NS_UNCONSTRAINEDSIZE);
 
     // Pass along the reflow command
-    nsSize              kidMaxElementSize;
     // Unless this is a fixed-layout table, then have the cell incrementally
     // update its maximum width. 
-    nsHTMLReflowMetrics cellMet(&kidMaxElementSize, isAutoLayout ? 
-                                                    NS_REFLOW_CALC_MAX_WIDTH : 0);
+    nsHTMLReflowMetrics cellMet(PR_TRUE,
+                                isAutoLayout ? NS_REFLOW_CALC_MAX_WIDTH : 0);
     GET_PIXELS_TO_TWIPS(aPresContext, p2t);
     nsTableCellReflowState kidRS(aPresContext, aReflowState, aNextFrame, cellAvailSize, 
                                  aReflowState.reason);
-    InitChildReflowState(*aPresContext, cellAvailSize, aTableFrame.IsBorderCollapse(), p2t, kidRS); 
+    // If the table will intialize the strategy (and balance) or balance, make the computed 
+    // width unconstrained. This avoids having the cell block compute a bogus max width 
+    // which will bias the balancing. Leave the avail width alone, since it is a best guess.
+    // After the table balances, the cell will get reflowed with the correct computed width.
+    PRBool resetComputedWidth = aTableFrame.NeedStrategyInit() || aTableFrame.NeedStrategyBalance();
+    InitChildReflowState(*aPresContext, cellAvailSize, aTableFrame.IsBorderCollapse(), 
+                         p2t, kidRS, resetComputedWidth); 
 
     // Remember the current desired size, we'll need it later
     nscoord oldCellMinWidth     = cellFrame->GetPass1MaxElementWidth();
@@ -1302,7 +1318,7 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     nscoord initCellDesDescent = cellMet.descent;
     
     // cache the max-elem and maximum widths
-    cellFrame->SetPass1MaxElementWidth(cellMet.width, kidMaxElementSize.width);
+    cellFrame->SetPass1MaxElementWidth(cellMet.width, cellMet.mMaxElementWidth);
     cellFrame->SetMaximumWidth(cellMet.mMaximumWidth);
 
     // Calculate the cell's actual size given its pass2 size. This function
@@ -1330,7 +1346,7 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
         PRInt32 rowSpan = aTableFrame.GetEffectiveRowSpan(*cellFrame);
         if (rowSpan == 1) {
          tallestCellGotShorter = 
-           TallestCellGotShorter(oldCellDesAscent, cellMet.descent, mMaxCellDescent);
+           TallestCellGotShorter(oldCellDesDescent, cellMet.descent, mMaxCellDescent);
         }
       }
     }
@@ -1444,7 +1460,8 @@ nsTableRowFrame::Reflow(nsIPresContext*          aPresContext,
     break;
 
   case eReflowReason_Resize:
-  case eReflowReason_StyleChange:
+  case eReflowReason_StyleChange: 
+  case eReflowReason_Dirty:
     rv = ReflowChildren(aPresContext, aDesiredSize, aReflowState, *tableFrame, aStatus, PR_FALSE);
     break;
 
@@ -1533,11 +1550,16 @@ nsTableRowFrame::ReflowCellFrame(nsIPresContext*          aPresContext,
 
   ReflowChild(aCellFrame, aPresContext, desiredSize, cellReflowState,
               0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
-  aCellFrame->SizeTo(aPresContext, cellSize.width, aAvailableHeight);
+  PRBool fullyComplete = NS_FRAME_IS_COMPLETE(aStatus) && !NS_FRAME_IS_TRUNCATED(aStatus);
+
+  aCellFrame->SizeTo(aPresContext, cellSize.width, 
+                     (fullyComplete) ? aAvailableHeight : desiredSize.height);
 
   // XXX What happens if this cell has 'vertical-align: baseline' ?
   // XXX Why is it assumed that the cell's ascent hasn't changed ?
-  aCellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
+  if (fullyComplete) {
+    aCellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
+  }
   aCellFrame->DidReflow(aPresContext, nsnull, NS_FRAME_REFLOW_FINISHED);
 
   return desiredSize.height;
