@@ -67,6 +67,7 @@
 #include "nsIScrollableFrame.h"
 #include "nsCSSFrameConstructor.h"
 #include "nsGUIEvent.h"
+#include "nsIRootBox.h"
 
 #define NS_MENU_POPUP_LIST_INDEX   0
 
@@ -158,6 +159,16 @@ nsPopupSetFrame::Init(nsIPresContext*  aPresContext,
 {
   mPresContext = aPresContext; // Don't addref it.  Our lifetime is shorter.
   nsresult  rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+
+  nsIFrame *grandParent;
+  aParent->GetParent(&grandParent);
+  nsIRootBox *rootBox;
+  nsresult res = CallQueryInterface(grandParent, &rootBox);
+  NS_ASSERTION(NS_SUCCEEDED(res), "grandparent should be root box");
+  if (NS_SUCCEEDED(res)) {
+    rootBox->SetPopupSetFrame(this);
+  }
+
   return rv;
 }
 
@@ -177,6 +188,15 @@ nsPopupSetFrame::Destroy(nsIPresContext* aPresContext)
     }
   }
 
+  nsIFrame *grandParent;
+  mParent->GetParent(&grandParent);
+  nsIRootBox *rootBox;
+  nsresult res = CallQueryInterface(grandParent, &rootBox);
+  NS_ASSERTION(NS_SUCCEEDED(res), "grandparent should be root box");
+  if (NS_SUCCEEDED(res)) {
+    rootBox->SetPopupSetFrame(nsnull);
+  }
+
   return nsBoxFrame::Destroy(aPresContext);
 }
 
@@ -364,7 +384,9 @@ nsPopupSetFrame::ShowPopup(nsIContent* aElementContent, nsIContent* aPopupConten
 
   // determine if this menu is a context menu and flag it
   nsIFrame* activeChild = entry->mPopupFrame;
-  nsCOMPtr<nsIMenuParent> childPopup ( do_QueryInterface(activeChild) );
+  nsIMenuParent* childPopup = nsnull;
+  if (activeChild)
+    CallQueryInterface(activeChild, &childPopup);
   if ( childPopup && aPopupType == NS_LITERAL_STRING("context") )
     childPopup->SetIsContextMenu(PR_TRUE);
 
@@ -384,13 +406,35 @@ nsPopupSetFrame::HidePopup(nsIFrame* aPopup)
     return NS_OK; // No active popups
 
   nsPopupFrameList* entry = mPopupList->GetEntryByFrame(aPopup);
-  if (entry && entry->mCreateHandlerSucceeded)
+  if (!entry)
+    return NS_OK;
+
+  if (entry->mCreateHandlerSucceeded)
     ActivatePopup(entry, PR_FALSE);
+
+  if (entry->mElementContent && entry->mPopupType == NS_LITERAL_STRING("context")) {
+    // If we are a context menu, and if we are attached to a menupopup, then hiding us
+    // should also hide the parent menu popup.
+    nsCOMPtr<nsIAtom> tag;
+    entry->mElementContent->GetTag(*getter_AddRefs(tag));
+    if (tag && tag.get() == nsXULAtoms::menupopup) {
+      nsIFrame* popupFrame = nsnull;
+      nsCOMPtr<nsIPresShell> presShell;
+      mPresContext->GetShell(getter_AddRefs(presShell));
+      presShell->GetPrimaryFrameFor(entry->mElementContent, &popupFrame);
+      if (popupFrame) {
+        nsCOMPtr<nsIMenuParent> menuParent(do_QueryInterface(popupFrame));
+        if (menuParent)
+          menuParent->HideChain();
+      }
+    }
+  }
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPopupSetFrame::DestroyPopup(nsIFrame* aPopup)
+nsPopupSetFrame::DestroyPopup(nsIFrame* aPopup, PRBool aDestroyEntireChain)
 {
   if (!mPopupList)
     return NS_OK; // No active popups
@@ -401,6 +445,24 @@ nsPopupSetFrame::DestroyPopup(nsIFrame* aPopup)
     OpenPopup(entry, PR_FALSE);
     entry->mPopupType.SetLength(0);
   
+    if (aDestroyEntireChain && entry->mElementContent && entry->mPopupType == NS_LITERAL_STRING("context")) {
+      // If we are a context menu, and if we are attached to a menupopup, then destroying us
+      // should also dismiss the parent menu popup.
+      nsCOMPtr<nsIAtom> tag;
+      entry->mElementContent->GetTag(*getter_AddRefs(tag));
+      if (tag && tag.get() == nsXULAtoms::menupopup) {
+        nsIFrame* popupFrame = nsnull;
+        nsCOMPtr<nsIPresShell> presShell;
+        mPresContext->GetShell(getter_AddRefs(presShell));
+        presShell->GetPrimaryFrameFor(entry->mElementContent, &popupFrame);
+        if (popupFrame) {
+          nsCOMPtr<nsIMenuParent> menuParent(do_QueryInterface(popupFrame));
+          if (menuParent)
+            menuParent->DismissChain();
+        }
+      }
+    }
+
     // clear things out for next time
     entry->mCreateHandlerSucceeded = PR_FALSE;
     entry->mElementContent = nsnull;
@@ -438,7 +500,9 @@ nsPopupSetFrame::OpenPopup(nsPopupFrameList* aEntry, PRBool aActivateFlag)
 
     // register the rollup listeners, etc, but not if we're a tooltip
     nsIFrame* activeChild = aEntry->mPopupFrame;
-    nsCOMPtr<nsIMenuParent> childPopup = do_QueryInterface(activeChild);
+    nsIMenuParent* childPopup = nsnull;
+    if (activeChild)
+      CallQueryInterface(activeChild, &childPopup);
     if (aEntry->mPopupType != NS_LITERAL_STRING("tooltip"))
       UpdateDismissalListener(childPopup);
     
@@ -462,7 +526,9 @@ nsPopupSetFrame::OpenPopup(nsPopupFrameList* aEntry, PRBool aActivateFlag)
     }
     
     // Remove any keyboard navigators
-    nsCOMPtr<nsIMenuParent> childPopup = do_QueryInterface(aEntry->mPopupFrame);
+    nsIMenuParent* childPopup = nsnull;
+    if (aEntry->mPopupFrame)
+      CallQueryInterface(aEntry->mPopupFrame, &childPopup);
     if (childPopup)
       childPopup->RemoveKeyboardNavigator();
 
