@@ -47,7 +47,6 @@
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsHTMLContainerFrame.h"
-#include "nsHTMLIIDs.h"
 #include "nsCSSRendering.h"
 #include "nsIScrollableView.h"
 #include "nsWidgetsCID.h"
@@ -57,10 +56,11 @@
 #include "nsBoxLayoutState.h"
 #include "nsIBoxToBlockAdaptor.h"
 #include "nsIScrollbarMediator.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIPresState.h"
 #include "nsButtonBoxFrame.h"
-#include "nsITimerCallback.h"
+#include "nsITimer.h"
 #include "nsRepeatService.h"
 
 static NS_DEFINE_IID(kWidgetCID, NS_CHILD_CID);
@@ -243,63 +243,29 @@ nsScrollBoxFrame::CreateScrollingView(nsIPresContext* aPresContext)
   GetScrollingParentView(aPresContext, parent, &parentView);
  
   // Get the view manager
-  nsIViewManager* viewManager;
-  parentView->GetViewManager(viewManager);
+  nsCOMPtr<nsIViewManager> viewManager;
+  parentView->GetViewManager(*getter_AddRefs(viewManager));
 
- 
   // Create the scrolling view
-  nsresult rv = nsComponentManager::CreateInstance(kScrollBoxViewCID, 
-                                             nsnull, 
-                                             NS_GET_IID(nsIView), 
-                                             (void **)&view);
-
-
- 
-  
-  
-
-  if (NS_OK == rv) {
-    const nsStyleDisplay* display = (const nsStyleDisplay*)
-      mStyleContext->GetStyleData(eStyleStruct_Display);
-    const nsStylePosition* position = (const nsStylePosition*)
-      mStyleContext->GetStyleData(eStyleStruct_Position);
-    const nsStyleBorder*  borderStyle = (const nsStyleBorder*)
-      mStyleContext->GetStyleData(eStyleStruct_Border);
-    const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-
-    // Get the z-index
-    PRInt32 zIndex = 0;
-    PRBool autoZIndex = PR_FALSE;
-    if (eStyleUnit_Integer == position->mZIndex.GetUnit()) {
-      zIndex = position->mZIndex.GetIntValue();
-    } else if (eStyleUnit_Auto == position->mZIndex.GetUnit()) {
-      autoZIndex = PR_TRUE;
-    }
-
+  nsresult rv = CallCreateInstance(kScrollBoxViewCID, &view);
+  if (NS_SUCCEEDED(rv)) {
     // Initialize the scrolling view
-    view->Init(viewManager, mRect, parentView, vis->IsVisibleOrCollapsed() ?
-               nsViewVisibility_kShow : nsViewVisibility_kHide);
+    view->Init(viewManager, mRect, parentView);
 
-    viewManager->SetViewZIndex(view, autoZIndex, zIndex);
+    SyncFrameViewProperties(aPresContext, this, mStyleContext, view);
 
     // Insert the view into the view hierarchy
     // XXX Put view last in document order until we know how to do better
     viewManager->InsertChild(parentView, view, nsnull, PR_TRUE);
 
-    // Set the view's opacity
-    viewManager->SetViewOpacity(view, vis->mOpacity);
-
-    // Because we only paint the border and we don't paint a background,
-    // inform the view manager that we have transparent content
-    viewManager->SetViewContentTransparency(view, PR_TRUE);
-
+    const nsStyleDisplay* display;
+    ::GetStyleData(mStyleContext, &display);
     // If it's fixed positioned, then create a widget too
     CreateScrollingViewWidget(view, display);
 
     // Get the nsIScrollableView interface
     nsIScrollableView* scrollingView;
-    view->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollingView);
+    CallQueryInterface(view, &scrollingView);
 
     scrollingView->SetScrollPreference(nsScrollPreference_kNeverScroll);
 
@@ -308,6 +274,8 @@ nsScrollBoxFrame::CreateScrollingView(nsIPresContext* aPresContext)
       scrollingView->CreateScrollControls(); 
     }
 
+    const nsStyleBorder* borderStyle;
+    ::GetStyleData(mStyleContext, &borderStyle);
     // Set the scrolling view's insets to whatever our border is
     nsMargin border;
     if (!borderStyle->GetBorder(border)) {
@@ -319,8 +287,6 @@ nsScrollBoxFrame::CreateScrollingView(nsIPresContext* aPresContext)
     // Remember our view
     SetView(aPresContext, view);
   }
-
-  NS_RELEASE(viewManager);
   return rv;
 }
 
@@ -426,20 +392,16 @@ nsScrollBoxFrame::DoLayout(nsBoxLayoutState& aState)
      nsIFrame* frame;
      kid->GetFrame(&frame);
      frame->GetView(presContext, &view);
-     nsCOMPtr<nsIViewManager> vm;
-     view->GetViewManager(*getter_AddRefs(vm));
+
      nsRect r(0, 0, childRect.width, childRect.height);
-     nsRect bnds;
-     view->GetBounds(bnds);
-     if (bnds != r) {
-       vm->ResizeView(view, r);
-     }
+     nsContainerFrame::SyncFrameViewAfterReflow(presContext, frame, view, &r,
+                                                NS_FRAME_NO_MOVE_VIEW);
   }
 
   nsIScrollableView* scrollingView;
   nsIView*           view;
   GetView(presContext, &view);
-  if (NS_SUCCEEDED(view->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollingView))) {
+  if (NS_SUCCEEDED(CallQueryInterface(view, &scrollingView))) {
     scrollingView->ComputeScrollOffsets(PR_TRUE);
   }
 
@@ -516,7 +478,8 @@ nsScrollBoxFrame::DoLayout(nsBoxLayoutState& aState)
     if (!view)
       return NS_OK; // don't freak out if we have no view
 
-    nsCOMPtr<nsIScrollableView> scrollingView(do_QueryInterface(view));
+    nsIScrollableView* scrollingView;
+    CallQueryInterface(view, &scrollingView);
     if (scrollingView) {
       nscoord x = 0;
       nscoord y = 0;
@@ -661,19 +624,12 @@ nsScrollBoxFrame::Paint(nsIPresContext*      aPresContext,
   }
 
   // Paint our children
-  nsresult rv = nsBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
-                                 aWhichLayer);
+  nsBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 
   // Call nsFrame::Paint to draw selection border when appropriate
   return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 }
 
-PRIntn
-nsScrollBoxFrame::GetSkipSides() const
-{
-  return 0;
-}
-
 nsresult 
 nsScrollBoxFrame::GetContentOf(nsIContent** aContent)
 {
@@ -735,7 +691,7 @@ nsScrollBoxFrame::SaveState(nsIPresContext* aPresContext,
 
   PRInt32 x,y;
   nsIScrollableView* scrollingView;
-  res = view->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollingView);
+  res = CallQueryInterface(view, &scrollingView);
   NS_ENSURE_SUCCESS(res, res);
   scrollingView->GetScrollPosition(x,y);
 
@@ -832,7 +788,8 @@ nsScrollBoxFrame::RestoreState(nsIPresContext* aPresContext,
       if (!view)
         return NS_ERROR_FAILURE;
 
-      nsCOMPtr<nsIScrollableView> scrollingView(do_QueryInterface(view));
+      nsIScrollableView* scrollingView;
+      CallQueryInterface(view, &scrollingView);
       if (scrollingView)
         scrollingView->GetScrollPosition(mLastPos.x, mLastPos.y);
     }
@@ -873,8 +830,7 @@ public:
               nsIStyleContext* aContext,
               nsIFrame*        aPrevInFlow);
 
-  NS_IMETHOD_(void) Notify(nsITimer *timer);
-
+  NS_DECL_NSITIMERCALLBACK
   nsIPresContext* mPresContext;
   
 };
@@ -940,10 +896,11 @@ nsAutoRepeatBoxFrame::HandleEvent(nsIPresContext* aPresContext,
   return nsButtonBoxFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
 }
 
-NS_IMETHODIMP_(void) 
+NS_IMETHODIMP
 nsAutoRepeatBoxFrame::Notify(nsITimer *timer)
 {
   MouseClicked(mPresContext, nsnull);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
