@@ -20,6 +20,10 @@
 
 /* This is where functions related to the standalone message window are kept */
 
+// from MailNewsTypes.h
+const nsMsgKey_None = 0xFFFFFFFF;
+const nsMsgViewIndex_None = 0xFFFFFFFF;
+
 /* globals for a particular window */
 
 var compositeDataSourceContractID        = datasourceContractIDPrefix + "composite-datasource";
@@ -29,90 +33,173 @@ var gCurrentMessageUri;
 var gCurrentFolderUri;
 var gThreadPaneCommandUpdater = null;
 var gCurrentMessageIsDeleted = false;
+var gNextMessageViewIndexAfterDelete = -2;
+var gCurrentFolderToRerootForStandAlone;
+var gRerootOnFolderLoadForStandAlone = false;
+var gNextMessageAfterLoad = null;
 
 // the folderListener object
 var folderListener = {
   OnItemAdded: function(parentItem, item, view) {},
 
-	OnItemRemoved: function(parentItem, item, view)
-	{
-		var parentFolderResource = parentItem.QueryInterface(Components.interfaces.nsIRDFResource);
-		if(!parentFolderResource)
-			return;
+  OnItemRemoved: function(parentItem, item, view) {
+    var parentFolderResource = parentItem.QueryInterface(Components.interfaces.nsIRDFResource);
+    if (!parentFolderResource)
+      return;
 
-		var parentURI = parentFolderResource.Value;
-		if(parentURI != gCurrentFolderUri)
-			return;
+    var parentURI = parentFolderResource.Value;
+    if (parentURI != gCurrentFolderUri)
+      return;
 
-		var deletedMessageHdr = item.QueryInterface(Components.interfaces.nsIMsgDBHdr);
+    var deletedMessageHdr = item.QueryInterface(Components.interfaces.nsIMsgDBHdr);
     if (extractMsgKeyFromURI() == deletedMessageHdr.messageKey)
-			gCurrentMessageIsDeleted = true;
-	},
+      gCurrentMessageIsDeleted = true;
+  },
 
-	OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
-
-	OnItemIntPropertyChanged: function(item, property, oldValue, newValue)
-	{
-	},
-
-	OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
+  OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
+  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) { 
+    var currentLoadedFolder = GetLoadedMsgFolder();
+    if (!currentLoadedFolder) return;
+    var currentURI = currentLoadedFolder.URI;
 
+    //if we don't have a folder loaded, don't bother.
+    if (currentURI) {
+      if (property.GetUnicode() == "TotalMessages" || property.GetUnicode() == "TotalUnreadMessages") {
+        var folder = item.QueryInterface(Components.interfaces.nsIMsgFolder);
+        if (folder) {
+          var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource); 
+          if (folderResource) {
+            var folderURI = folderResource.Value;
+            if (currentURI == folderURI) {
+              UpdateStandAloneMessageCounts();
+            }
+          }
+        }
+      }      
+    }
+  },
+  OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
   OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){},
-	OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
+  OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
 
   OnItemEvent: function(folder, event) {
-		  if (event.GetUnicode() == "DeleteOrMoveMsgCompleted") {
-			  HandleDeleteOrMoveMsgCompleted(folder);
-		  }     
-      else if (event.GetUnicode() == "DeleteOrMoveMsgFailed") {
-            HandleDeleteOrMoveMsgFailed(folder);
+    var eventType = event.GetUnicode();
+
+    if (eventType == "DeleteOrMoveMsgCompleted")
+      HandleDeleteOrMoveMsgCompleted(folder);
+    else if (eventType == "DeleteOrMoveMsgFailed")
+      HandleDeleteOrMoveMsgFailed(folder);
+    else if (eventType == "FolderLoaded") {
+      if (folder) {
+        var uri = folder.URI;
+        if (uri == gCurrentFolderToRerootForStandAlone) {
+          gCurrentFolderToRerootForStandAlone = null;
+          var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
+          if (msgFolder) {
+            msgFolder.endFolderLoading();
+            if (gRerootOnFolderLoadForStandAlone) {
+              RerootFolderForStandAlone(uri);
+            }
+          }   
+        }
       }
     }
+  }   
 }
 
+var messagepaneObserver = {
+
+  canHandleMultipleItems: false,
+
+  onDrop: function (aEvent, aData, aDragSession)
+  {
+    var sourceUri = aData.data; 
+    if (sourceUri != gCurrentMessageUri)
+    {
+      SelectFolder(GetMsgHdrFromUri(sourceUri).folder.URI);
+      SelectMessage(sourceUri);
+    }
+  },
+ 
+  onDragOver: function (aEvent, aFlavour, aDragSession)
+  {
+    var messagepanebox = document.getElementById("messagepanebox");
+    messagepanebox.setAttribute("dragover", "true");
+  },
+ 
+  onDragExit: function (aEvent, aDragSession)
+  {
+    var messagepanebox = document.getElementById("messagepanebox");
+    messagepanebox.removeAttribute("dragover");
+  },
+
+  canDrop: function(aEvent, aDragSession)  //allow drop from mail:3pane window only - 4xp
+  {
+    var doc = aDragSession.sourceNode.ownerDocument;
+    var elem = doc.getElementById("messengerWindow");
+    return (elem && (elem.getAttribute("windowtype") == "mail:3pane"));
+  },
+  
+  getSupportedFlavours: function ()
+  {
+    var flavourSet = new FlavourSet();
+    flavourSet.appendFlavour("text/x-moz-message");
+    return flavourSet;
+  }
+};
+
 function nsMsgDBViewCommandUpdater()
 {}
 
+function UpdateStandAloneMessageCounts()
+{
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:updateStandAloneMessageCounts", "");
+}
+
 nsMsgDBViewCommandUpdater.prototype = 
 {
   updateCommandStatus : function()
     {
       // the back end is smart and is only telling us to update command status
       // when the # of items in the selection has actually changed.
-		  document.commandDispatcher.updateCommands('mail-toolbar');
+      UpdateMailToolbar("dbview, std alone window");
     },
 
-  displayMessageChanged : function(aFolder, aSubject)
+  displayMessageChanged : function(aFolder, aSubject, aKeywords)
   {
     setTitleFromFolder(aFolder, aSubject);
     gCurrentMessageUri = gDBView.URIForFirstSelectedMessage;
+    UpdateStandAloneMessageCounts();
+    SetKeywords(aKeywords);
   },
 
   QueryInterface : function(iid)
   {
-    if(iid.equals(Components.interfaces.nsIMsgDBViewCommandUpdater))
-	    return this;
+    if (iid.equals(Components.interfaces.nsIMsgDBViewCommandUpdater) ||
+        iid.equals(Components.interfaces.nsISupports))
+      return this;
 	  
     throw Components.results.NS_NOINTERFACE;
-    return null;
   }
 }
 
 function HandleDeleteOrMoveMsgCompleted(folder)
 {
-	dump("In HandleDeleteOrMoveMsgCompleted\n");
 	var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-	if(!folderResource)
+	if (!folderResource)
 		return;
 
 	var folderUri = folderResource.Value;
-	if((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
+	if ((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
 	{
+    gDBView.onDeleteCompleted(true);
     gCurrentMessageIsDeleted = false;
-    if (gNextMessageViewIndexAfterDelete != -1) 
+    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
     {
       var nextMstKey = gDBView.getKeyAt(gNextMessageViewIndexAfterDelete);
-      if (nextMstKey != -1) {
+      if (nextMstKey != nsMsgKey_None) {
         gDBView.loadMessageByMsgKey(nextMstKey);
       }
       else {
@@ -130,23 +217,24 @@ function HandleDeleteOrMoveMsgCompleted(folder)
 function HandleDeleteOrMoveMsgFailed(folder)
 {
   var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-  if(!folderResource)
+  if (!folderResource)
      return;
 
   var folderUri = folderResource.Value;
-  if((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
-  {
+  gDBView.onDeleteCompleted(false);
+  if ((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
     gCurrentMessageIsDeleted = false;
-  }	
 }
 
 function OnLoadMessageWindow()
 {
 	HideMenus();
-  	AddMailOfflineObserver();
+  AddMailOfflineObserver();
 	CreateMailWindowGlobals();
 	CreateMessageWindowGlobals();
-	verifyAccounts();
+  AddToolBarPrefListener();
+  ShowHideToolBarButtons()
+	verifyAccounts(null);
 
 	InitMsgWindow();
 
@@ -156,37 +244,59 @@ function OnLoadMessageWindow()
 	OnLoadMsgHeaderPane();
 
   try {
-    mailSession.AddFolderListener(folderListener);
-	} catch (ex) {
-    dump("Error adding to session\n");
+    var nsIFolderListener = Components.interfaces.nsIFolderListener;
+    var notifyFlags = nsIFolderListener.removed | nsIFolderListener.event | nsIFolderListener.intPropertyChanged;
+    mailSession.AddFolderListener(folderListener, notifyFlags);
+  } catch (ex) {
+    dump("Error adding to session: " +ex + "\n");
   }
 
   var originalView = null;
 
-	if(window.arguments)
+	if (window.arguments)
 	{
-		if(window.arguments[0])
-		{
+		if (window.arguments[0])
 			gCurrentMessageUri = window.arguments[0];
-		}
 		else
-		{
 			gCurrentMessageUri = null;
-		}
 
-		if(window.arguments[1])
-		{
+		if (window.arguments[1])
 			gCurrentFolderUri = window.arguments[1];
-		}
 		else
-		{
 			gCurrentFolderUri = null;
-		}
 
     if (window.arguments[2])
       originalView = window.arguments[2];      
 	}	
 
+  CreateView(originalView)
+ 
+  setTimeout(OnLoadMessageWindowDelayed, 0);
+  
+  SetupCommandUpdateHandlers();
+}
+
+function OnLoadMessageWindowDelayed()
+{
+  var msgKey = extractMsgKeyFromURI(gCurrentMessageUri); 
+  gDBView.loadMessageByMsgKey(msgKey); 
+  gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete; 
+  UpdateStandAloneMessageCounts();
+   
+  // set focus to the message pane
+  window.content.focus();
+
+  // since we just changed the pane with focus we need to update the toolbar to reflect this
+  // XXX TODO
+  // can we optimize
+  // and just update cmd_delete and button_delete?
+  UpdateMailToolbar("focus");
+}
+
+function CreateView(originalView)
+{
+  var msgFolder = GetLoadedMsgFolder();
+
   // extract the sort type, the sort order, 
   var sortType;
   var sortOrder;
@@ -200,23 +310,37 @@ function OnLoadMessageWindow()
     sortType = originalView.sortType;
     sortOrder = originalView.sortOrder;
   }
-  
-  var msgFolder = GetLoadedMsgFolder();
-  CreateBareDBView(msgFolder,viewType, viewFlags, sortType, sortOrder); // create a db view for 
-
-  if (gCurrentMessageUri) {
-    SetUpToolbarButtons(gCurrentMessageUri);
+  else if (msgFolder)
+  {
+    var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
+    if (msgDatabase)
+    {
+      var dbFolderInfo = msgDatabase.dBFolderInfo;
+      sortType = dbFolderInfo.sortType;
+      sortOrder = dbFolderInfo.sortOrder;
+      viewFlags = dbFolderInfo.viewFlags;
+      viewType = dbFolderInfo.viewType;
+      msgDatabase = null;
+      dbFolderInfo = null;
+   }
   }
-  else if (gCurrentFolderUri) {
-    SetUpToolbarButtons(gCurrentFolderUri);
-  }
- 
-  setTimeout("var msgKey = extractMsgKeyFromURI(gCurrentMessageUri); gDBView.loadMessageByMsgKey(msgKey); gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete;", 0);
 
-  SetupCommandUpdateHandlers();
-  var messagePaneFrame = top.frames['messagepane'];
-  if(messagePaneFrame)
-	messagePaneFrame.focus();
+  // create a db view
+  CreateBareDBView(originalView, msgFolder, viewType, viewFlags, sortType, sortOrder); 
+
+  var uri;
+  if (gCurrentMessageUri)
+    uri = gCurrentMessageUri;
+  else if (gCurrentFolderUri)
+    uri = gCurrentFolderUri;
+  else
+    uri = null;
+
+  SetUpToolbarButtons(uri);
+
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
 }
 
 function extractMsgKeyFromURI()
@@ -236,53 +360,64 @@ function extractMsgKeyFromURI()
 function HideMenus()
 {
 	var message_menuitem=document.getElementById('menu_showMessage');
-	if(message_menuitem)
+	if (message_menuitem)
 		message_menuitem.setAttribute("hidden", "true");
 
+	var showSearchToolbar = document.getElementById('menu_showSearchToolbar');
+	if (showSearchToolbar)
+		showSearchToolbar.setAttribute("hidden", "true");
+
+	var showSearch_showMessage_Separator = document.getElementById('menu_showSearch_showMessage_Separator');
+	if (showSearch_showMessage_Separator)
+		showSearch_showMessage_Separator.setAttribute("hidden", "true");
+
 	var expandOrCollapseMenu = document.getElementById('menu_expandOrCollapse');
-	if(expandOrCollapseMenu)
+	if (expandOrCollapseMenu)
 		expandOrCollapseMenu.setAttribute("hidden", "true");
 
 	var renameFolderMenu = document.getElementById('menu_renameFolder');
-	if(renameFolderMenu)
+	if (renameFolderMenu)
 		renameFolderMenu.setAttribute("hidden", "true");
 
 	var viewMessagesMenu = document.getElementById('viewMessagesMenu');
-	if(viewMessagesMenu)
+	if (viewMessagesMenu)
 		viewMessagesMenu.setAttribute("hidden", "true");
 
 	var openMessageMenu = document.getElementById('openMessageWindowMenuitem');
-	if(openMessageMenu)
+	if (openMessageMenu)
 		openMessageMenu.setAttribute("hidden", "true");
 
 	var viewSortMenu = document.getElementById('viewSortMenu');
-	if(viewSortMenu)
+	if (viewSortMenu)
 		viewSortMenu.setAttribute("hidden", "true");
 
 	var viewThreadedMenu = document.getElementById('menu_showThreads');
-	if(viewThreadedMenu)
+	if (viewThreadedMenu)
 		viewThreadedMenu.setAttribute("hidden", "true");
 
 	var emptryTrashMenu = document.getElementById('menu_emptyTrash');
-	if(emptryTrashMenu)
+	if (emptryTrashMenu)
 		emptryTrashMenu.setAttribute("hidden", "true");
 
 	var menuProperties = document.getElementById('menu_properties');
-	if(menuProperties)
+	if (menuProperties)
 		menuProperties.setAttribute("hidden", "true");
 
 	var compactFolderMenu = document.getElementById('menu_compactFolder');
-	if(compactFolderMenu)
+	if (compactFolderMenu)
 		compactFolderMenu.setAttribute("hidden", "true");
 
 	var trashSeparator = document.getElementById('trashMenuSeparator');
-	if(trashSeparator)
+	if (trashSeparator)
 		trashSeparator.setAttribute("hidden", "true");
-
 }
 
 function OnUnloadMessageWindow()
 {
+  RemoveToolBarPrefListener();
+	// FIX ME - later we will be able to use onunload from the overlay
+	OnUnloadMsgHeaderPane();
+
 	OnMailWindowUnload();
 }
 
@@ -304,10 +439,9 @@ function GetSelectedMsgFolders()
 {
 	var folderArray = new Array(1);
 	var msgFolder = GetLoadedMsgFolder();
-	if(msgFolder)
-	{
+	if (msgFolder)
 		folderArray[0] = msgFolder;	
-	}
+
 	return folderArray;
 }
 
@@ -318,7 +452,7 @@ function GetFirstSelectedMessage()
 
 function GetNumSelectedMessages()
 {
-	if(gCurrentMessageUri)
+	if (gCurrentMessageUri)
 		return 1;
 	else
 		return 0;
@@ -328,24 +462,32 @@ function GetSelectedMessages()
 {
 	var messageArray = new Array(1);
 	var message = GetLoadedMessage();
-	if(message) {
+	if (message)
 		messageArray[0] = message;	
-	}
+
 	return messageArray;
 }
 
+function GetSelectedIndices(dbView)
+{
+  try {
+    var indicesArray = {}; 
+    var length = {};
+    dbView.getIndicesForSelection(indicesArray,length);
+    return indicesArray.value;
+  }
+  catch (ex) {
+    dump("ex = " + ex + "\n");
+    return null;
+  }
+}
+
 function GetLoadedMsgFolder()
 {
-	if(gCurrentFolderUri)
-	{
-		var folderResource = RDF.GetResource(gCurrentFolderUri);
-		if(folderResource)
-		{
-			var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-			return msgFolder;
-		}
-	}
-	return null;
+	if (gCurrentFolderUri)
+		return RDF.GetResource(gCurrentFolderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
+	else
+    return null;
 }
 
 function GetLoadedMessage()
@@ -358,7 +500,7 @@ function ClearMessageSelection()
 {
 	gCurrentMessageUri = null;
 	gCurrentFolderUri = null;
-  document.commandDispatcher.updateCommands('mail-toolbar');	
+  UpdateMailToolbar("clear msg, std alone window");
 }
 
 function GetCompositeDataSource(command)
@@ -373,9 +515,79 @@ function SetNextMessageAfterDelete()
 
 function SelectFolder(folderUri)
 {
-	gCurrentFolderUri = folderUri;
+  if (folderUri == gCurrentFolderUri)
+    return;
+
+  var msgfolder = RDF.GetResource(folderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
+  if (!msgfolder || msgfolder.isServer)
+    return;
+
+  // close old folder view
+  var dbview = GetDBView();  
+  if (dbview)
+    dbview.close(); 
+
+  gCurrentFolderToRerootForStandAlone = folderUri;
+
+  if (msgfolder.manyHeadersToDownload)
+  {
+    gRerootOnFolderLoadForStandAlone = true;
+    try
+    {
+      msgfolder.startFolderLoading();
+      msgfolder.updateFolder(msgWindow);
+    }
+    catch(ex)
+    {
+      dump("Error loading with many headers to download: " + ex + "\n");
+    }
+  }
+  else
+  {
+    RerootFolderForStandAlone(folderUri);
+    gRerootOnFolderLoadForStandAlone = false;
+    msgfolder.startFolderLoading();
+
+    //Need to do this after rerooting folder.  Otherwise possibility of receiving folder loaded
+    //notification before folder has actually changed.
+    msgfolder.updateFolder(msgWindow);
+  }    
 }
 
+function RerootFolderForStandAlone(uri)
+{
+  gCurrentFolderUri = uri;
+
+  // create new folder view
+  CreateView(null);
+  
+  // now do the work to load the appropriate message
+  if (gNextMessageAfterLoad) {
+    var type = gNextMessageAfterLoad;
+    gNextMessageAfterLoad = null;
+    LoadMessageByNavigationType(type);
+  }
+  
+  SetUpToolbarButtons(gCurrentFolderUri);
+  
+  UpdateMailToolbar("reroot folder in stand alone window");
+  
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
+} 
+
+function GetMsgHdrFromUri(messageUri)
+{
+  return messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri);
+}
+
+function SelectMessage(messageUri)
+{
+  var msgHdr = GetMsgHdrFromUri(messageUri);
+  gDBView.loadMessageByMsgKey(msgHdr.messageKey);
+}
+ 
 function ReloadMessage()
 {
   gDBView.reloadMessage();
@@ -423,13 +635,20 @@ var MessageWindowController =
 			case "cmd_forwardInline":
 			case "cmd_forwardAttachment":
 			case "cmd_editAsNew":
+      case "cmd_createFilterFromPopup":
+      case "cmd_createFilterFromMenu":
 			case "cmd_delete":
+      case "cmd_undo":
+      case "cmd_redo":
       case "cmd_killThread":
       case "cmd_watchThread":
 			case "button_delete":
+      case "button_junk":
 			case "cmd_shiftDelete":
       case "button_print":
 			case "cmd_print":
+			case "cmd_printpreview":
+		  case "cmd_printSetup":
 			case "cmd_saveAsFile":
 			case "cmd_saveAsTemplate":
 			case "cmd_viewPageSource":
@@ -440,16 +659,21 @@ var MessageWindowController =
 			case "cmd_getNextNMessages":
 			case "cmd_find":
 			case "cmd_findAgain":
+			case "cmd_findPrev":
+      case "cmd_search":
       case "button_mark":
 			case "cmd_markAsRead":
 			case "cmd_markAllRead":
 			case "cmd_markThreadAsRead":
 			case "cmd_markAsFlagged":
+      case "cmd_label0":
+      case "cmd_label1":
+      case "cmd_label2":
+      case "cmd_label3":
+      case "cmd_label4":
+      case "cmd_label5":
       case "button_file":
 			case "cmd_file":
-      case "cmd_synchronizeOffline":
-			case "cmd_downloadFlagged":
-			case "cmd_downloadSelected":
 			case "cmd_settingsOffline":
 			case "cmd_nextMsg":
       case "button_next":
@@ -460,6 +684,10 @@ var MessageWindowController =
 			case "cmd_previousUnreadMsg":
 			case "cmd_previousFlaggedMsg":
 				return true;
+      case "cmd_synchronizeOffline":
+			case "cmd_downloadFlagged":
+			case "cmd_downloadSelected":
+        return CheckOnline();
 			default:
 				return false;
 		}
@@ -467,27 +695,22 @@ var MessageWindowController =
 
 	isCommandEnabled: function(command)
 	{
-    var enabled = new Object();
-    var checkStatus = new Object();
 		switch ( command )
 		{
+      case "cmd_createFilterFromPopup":
+      case "cmd_createFilterFromMenu":
+        var loadedFolder = GetLoadedMsgFolder();
+        if (!(loadedFolder && loadedFolder.server.canHaveFilters))
+          return false;
 			case "cmd_delete":
-				if (isNewsURI(gCurrentMessageUri))
-				{
-					goSetMenuValue(command, 'valueNewsMessage');
-					goSetAccessKey(command, 'valueNewsMessageAccessKey');
-				}
-				else
-				{
-					goSetMenuValue(command, 'valueMessage');
-					goSetAccessKey(command, 'valueMessageAccessKey');
-				}
-
-				if (gDBView)
-				{
-					gDBView.getCommandStatus(nsMsgViewCommandType.deleteMsg, enabled, checkStatus);
-					return enabled.value;
-				}
+        UpdateDeleteCommand();
+        // fall through
+			case "button_delete":
+			case "cmd_shiftDelete":
+        var loadedFolder = GetLoadedMsgFolder();
+        return gCurrentMessageUri && loadedFolder && (loadedFolder.canDeleteMessages || isNewsURI(gCurrentFolderUri));
+      case "button_junk":
+        return (!isNewsURI(gCurrentFolderUri));
 			case "cmd_reply":
 			case "button_reply":
 			case "cmd_replySender":
@@ -499,10 +722,8 @@ var MessageWindowController =
 			case "cmd_forwardInline":
 			case "cmd_forwardAttachment":
 			case "cmd_editAsNew":
-
-			case "button_delete":
-			case "cmd_shiftDelete":
 			case "cmd_print":
+			case "cmd_printpreview":
       case "button_print":
 			case "cmd_saveAsFile":
 			case "cmd_saveAsTemplate":
@@ -513,21 +734,31 @@ var MessageWindowController =
 			case "cmd_markAsRead":
 			case "cmd_markAllRead":
 			case "cmd_markThreadAsRead":
+      case "cmd_label0":
+      case "cmd_label1":
+      case "cmd_label2":
+      case "cmd_label3":
+      case "cmd_label4":
+      case "cmd_label5":
+        return(true);
 			case "cmd_markAsFlagged":
       case "button_file":
 			case "cmd_file":
 				return ( gCurrentMessageUri != null);
+			case "cmd_printSetup":
+			  return true;
 			case "cmd_getNewMessages":
       case "button_getNewMessages":
       case "cmd_getMsgsForAuthAccounts":
 				return IsGetNewMessagesEnabled();
 			case "cmd_getNextNMessages":
-				return IsGetNextNMessagesEnabled();
-			case "cmd_synchronizeOffline":
+				return IsGetNextNMessagesEnabled();		
 			case "cmd_downloadFlagged":
 			case "cmd_downloadSelected":
+      case "cmd_synchronizeOffline":
+                return CheckOnline();
 			case "cmd_settingsOffline":
-				return true;
+                return IsAccountOfflineEnabled();
 			case "cmd_close":
 			case "cmd_nextMsg":
       case "button_next":
@@ -537,7 +768,16 @@ var MessageWindowController =
 			case "cmd_previousUnreadMsg":
 				return true;
 			case "cmd_findAgain":
+			case "cmd_findPrev":
 				return MsgCanFindAgain();
+      case "cmd_search":
+        var loadedFolder = GetLoadedMsgFolder();
+        if (!loadedFolder)
+          return false;
+        return loadedFolder.server.canSearchMessages; 
+      case "cmd_undo":
+      case "cmd_redo":
+                return SetupUndoRedoCommand(command);
 			default:
 				return false;
 		}
@@ -559,6 +799,12 @@ var MessageWindowController =
 			case "cmd_getNewMessages":
 				MsgGetMessage();
 				break;
+            case "cmd_undo":
+                messenger.Undo(msgWindow);
+                break;
+            case "cmd_redo":
+                messenger.Redo(msgWindow);
+                break;
       case "cmd_getMsgsForAuthAccounts":
         MsgGetMessagesForAllAuthenticatedAccounts();
         break;
@@ -589,18 +835,32 @@ var MessageWindowController =
 			case "cmd_editAsNew":
 				MsgEditMessageAsNew();
 				break;
+      case "cmd_createFilterFromPopup":
+				break;// This does nothing because the createfilter is invoked from the popupnode oncommand.        
+      case "cmd_createFilterFromMenu":
+        MsgCreateFilter();
+				break;        
 			case "cmd_delete":
 				MsgDeleteMessageFromMessageWindow(false, false);
 				break;
 			case "cmd_shiftDelete":
 				MsgDeleteMessageFromMessageWindow(true, false);
 				break;
+      case "button_junk":
+        MsgJunk();
+        break;
 			case "button_delete":
 				MsgDeleteMessageFromMessageWindow(false, true);
 				break;
+		  case "cmd_printSetup":
+		    NSPrintSetup();
+		    break;
 			case "cmd_print":
 				PrintEnginePrint();
 				break;
+			case "cmd_printpreview":
+				PrintEnginePrintPreview();
+				break;
 			case "cmd_saveAsFile":
 				MsgSaveAsFile();
 				break;
@@ -617,8 +877,14 @@ var MessageWindowController =
 				MsgFind();
 				break;
 			case "cmd_findAgain":
-				MsgFindAgain();
+				MsgFindAgain(false);
 				break;
+			case "cmd_findPrev":
+				MsgFindAgain(true);
+				break;
+      case "cmd_search":
+        MsgSearchMessages();
+        break;
       case "button_mark":
 			case "cmd_markAsRead":
 				MsgMarkMsgAsRead(null);
@@ -632,6 +898,24 @@ var MessageWindowController =
 			case "cmd_markAsFlagged":
 				MsgMarkAsFlagged(null);
 				return;
+      case "cmd_label0":
+        gDBView.doCommand(nsMsgViewCommandType.label0);
+ 				return;
+      case "cmd_label1":
+        gDBView.doCommand(nsMsgViewCommandType.label1);
+        return; 
+      case "cmd_label2":
+        gDBView.doCommand(nsMsgViewCommandType.label2);
+        return; 
+      case "cmd_label3":
+        gDBView.doCommand(nsMsgViewCommandType.label3);
+        return; 
+      case "cmd_label4":
+        gDBView.doCommand(nsMsgViewCommandType.label4);
+        return; 
+      case "cmd_label5":
+        gDBView.doCommand(nsMsgViewCommandType.label5);
+        return; 
       case "cmd_downloadFlagged":
         MsgDownloadFlagged();
         return;
@@ -641,12 +925,13 @@ var MessageWindowController =
       case "cmd_synchronizeOffline":
         MsgSynchronizeOffline();
         return;
-			case "cmd_settingsOffline":
-				MsgSettingsOffline();
-				return;
-			case "cmd_nextUnreadMsg":
+      case "cmd_settingsOffline":
+        MsgSettingsOffline();
+        return;
+      case "cmd_nextUnreadMsg":
+      case "button_next":
         performNavigation(nsMsgNavigationType.nextUnreadMessage);
-				break;
+        break;
 			case "cmd_nextUnreadThread":      
         performNavigation(nsMsgNavigationType.nextUnreadThread);
 				break;
@@ -673,24 +958,38 @@ var MessageWindowController =
 	}
 };
 
-function performNavigation(type)
+function LoadMessageByNavigationType(type)
 {
   var resultId = new Object;
   var resultIndex = new Object;
   var threadIndex = new Object;
 
   gDBView.viewNavigate(type, resultId, resultIndex, threadIndex, true /* wrap */);
-  
+
   // if we found something....display it.
-  if ((resultId.value != nsMsgViewIndex_None) && (resultIndex.value != nsMsgViewIndex_None)) 
+  if ((resultId.value != nsMsgKey_None) && (resultIndex.value != nsMsgKey_None)) 
   {
     // load the message key
     gDBView.loadMessageByMsgKey(resultId.value);
-    return;
+    // if we changed folders, the message counts changed.
+    UpdateStandAloneMessageCounts();
+
+    // new message has been loaded
+    return true;
   }
+
+  // no message found to load
+  return false;
+}
    
-  // we need to span another folder 
-  CrossFolderNavigation(type, false);
+function performNavigation(type)
+{
+  // Try to load a message by navigation type if we can find
+  // the message in the same folder.
+  if (LoadMessageByNavigationType(type))
+    return;
+   
+  CrossFolderNavigation(type);
 }
 
 function SetupCommandUpdateHandlers()
@@ -698,12 +997,9 @@ function SetupCommandUpdateHandlers()
 	top.controllers.insertControllerAt(0, MessageWindowController);
 }
 
-function CommandUpdate_UndoRedo()
-{
-
-}
-
 function GetDBView()
 {
   return gDBView;
 }
+
+  
