@@ -521,7 +521,12 @@ char *nsIMAPGenericParser::CreateLiteral()
 	int32 charsReadSoFar = 0, currentLineLength = 0;
 	int32 bytesToCopy = 0;
 	
-	char *returnString = (char *) PR_Malloc(numberOfCharsInMessage + 1);
+	uint32 numBytes = numberOfCharsInMessage + 1;
+  NS_ASSERTION(numBytes, "overflow!");
+  if (!numBytes)
+    return nsnull;
+
+	char *returnString = (char *) PR_Malloc(numBytes);
 	
 	if (returnString)
 	{
