@@ -1,25 +1,40 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /*
 
@@ -29,6 +44,7 @@
 
 #include "nsCOMPtr.h"
 #include "nscore.h"
+#include "nsReadableUtils.h"
 #include "nsOutlookImport.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -49,8 +65,6 @@
 static NS_DEFINE_IID(kISupportsIID,        	NS_ISUPPORTS_IID);
 static NS_DEFINE_CID(kComponentManagerCID, 	NS_COMPONENTMANAGER_CID);
 static NS_DEFINE_CID(kSmtpServiceCID,		NS_SMTPSERVICE_CID); 
-static NS_DEFINE_CID(kMsgAccountMgrCID, NS_MSGACCOUNTMANAGER_CID);
-
 
 class OutlookSettings {
 public:
@@ -65,7 +79,7 @@ public:
 	static PRBool IdentityMatches( nsIMsgIdentity *pIdent, const char *pName, const char *pServer, const char *pEmail, const char *pReply, const char *pUserName);
 
 	static void SetSmtpServer( nsIMsgAccountManager *pMgr, nsIMsgAccount *pAcc, char *pServer, char *pUser);
-
+  static nsresult GetAccountName(HKEY hKey, char *defaultName, nsString &acctName);
 };
 
 
@@ -86,7 +100,6 @@ nsresult nsOutlookSettings::Create(nsIImportSettings** aImport)
 
 nsOutlookSettings::nsOutlookSettings()
 {
-    NS_INIT_REFCNT();
 }
 
 nsOutlookSettings::~nsOutlookSettings()
@@ -164,7 +177,7 @@ PRBool OutlookSettings::DoImport( nsIMsgAccount **ppAccount)
 	nsresult	rv;
 	
 	nsCOMPtr<nsIMsgAccountManager> accMgr = 
-	         do_GetService(kMsgAccountMgrCID, &rv);
+	         do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) {
 		IMPORT_LOG0( "*** Failed to create a account manager!\n");
 		return( PR_FALSE);
@@ -264,7 +277,20 @@ PRBool OutlookSettings::DoImport( nsIMsgAccount **ppAccount)
 	return( accounts != 0);
 }
 
-
+nsresult OutlookSettings::GetAccountName(HKEY hKey, char *defaultName, nsString &acctName)
+{
+  BYTE *pAccName = nsOutlookRegUtil::GetValueBytes( hKey, "Account Name");
+  nsresult rv = NS_OK;
+  if (pAccName) {
+    nsCOMPtr<nsIImportService> impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+    if (impSvc)
+      rv = impSvc->SystemStringToUnicode((const char *)pAccName, acctName);
+    nsOutlookRegUtil::FreeValueBytes( pAccName);
+  }
+  else
+    acctName.Assign(NS_ConvertASCIItoUCS2(defaultName));
+  return rv;
+}
 
 PRBool OutlookSettings::DoIMAPServer( nsIMsgAccountManager *pMgr, HKEY hKey, char *pServerName, nsIMsgAccount **ppAccount)
 {
@@ -291,21 +317,16 @@ PRBool OutlookSettings::DoIMAPServer( nsIMsgAccountManager *pMgr, HKEY hKey, cha
 			
 			IMPORT_LOG2( "Created IMAP server named: %s, userName: %s\n", pServerName, (char *)pBytes);
 
-			BYTE *pAccName = nsOutlookRegUtil::GetValueBytes( hKey, "Account Name");
 			nsString	prettyName;
-			if (pAccName) {
-				prettyName.AssignWithConversion((const char *)pAccName);
-				nsOutlookRegUtil::FreeValueBytes( pAccName);
-			}
-			else
-				prettyName.AssignWithConversion((const char *)pServerName);
-
-			PRUnichar *pretty = prettyName.ToNewUnicode();
-			
-			IMPORT_LOG1( "\tSet pretty name to: %S\n", pretty);
-
+      if (NS_SUCCEEDED(GetAccountName(hKey, pServerName, prettyName)))
+      {
+			PRUnichar *pretty = ToNewUnicode(prettyName);
+        if (pretty)
+        {
 			rv = in->SetPrettyName( pretty);
 			nsCRT::free( pretty);
+        }
+      }
 			
 			// We have a server, create an account.
 			nsCOMPtr<nsIMsgAccount>	account;
@@ -356,21 +377,16 @@ PRBool OutlookSettings::DoPOP3Server( nsIMsgAccountManager *pMgr, HKEY hKey, cha
 
 			IMPORT_LOG2( "Created POP3 server named: %s, userName: %s\n", pServerName, (char *)pBytes);
 
-			BYTE *pAccName = nsOutlookRegUtil::GetValueBytes( hKey, "Account Name");
 			nsString	prettyName;
-			if (pAccName) {
-				prettyName.AssignWithConversion((const char *)pAccName);
-				nsOutlookRegUtil::FreeValueBytes( pAccName);
-			}
-			else
-				prettyName.AssignWithConversion((const char *)pServerName);
-
-			PRUnichar *pretty = prettyName.ToNewUnicode();
-			
-			IMPORT_LOG1( "\tSet pretty name to: %S\n", pretty);
-
+      if (NS_SUCCEEDED(GetAccountName(hKey, pServerName, prettyName)))
+      {
+			PRUnichar *pretty = ToNewUnicode(prettyName);
+        if (pretty)
+        {
 			rv = in->SetPrettyName( pretty);
 			nsCRT::free( pretty);
+        }
+      }
 			
 			// We have a server, create an account.
 			nsCOMPtr<nsIMsgAccount>	account;
@@ -432,7 +448,7 @@ PRBool OutlookSettings::IdentityMatches( nsIMsgIdentity *pIdent, const char *pNa
 	if (ppIName) {
 		nsString name(ppIName);
 		nsCRT::free( ppIName);
-		pIName = name.ToNewCString();
+		pIName = ToNewCString(name);
 	}
 
 	// for now, if it's the same server and reply to and email then it matches
@@ -460,6 +476,7 @@ void OutlookSettings::SetIdentities( nsIMsgAccountManager *pMgr, nsIMsgAccount *
 	char *pEmail = (char *)nsOutlookRegUtil::GetValueBytes( hKey, "SMTP Email Address");
 	char *pReply = (char *)nsOutlookRegUtil::GetValueBytes( hKey, "SMTP Reply To Email Address");
 	char *pUserName = (char *)nsOutlookRegUtil::GetValueBytes( hKey, "SMTP User Name");
+  char *pOrgName = (char *)nsOutlookRegUtil::GetValueBytes( hKey, "SMTP Organization Name");
 
 	nsresult	rv;
 
@@ -469,10 +486,20 @@ void OutlookSettings::SetIdentities( nsIMsgAccountManager *pMgr, nsIMsgAccount *
 		nsCOMPtr<nsIMsgIdentity>	id;
 		rv = pMgr->CreateIdentity( getter_AddRefs( id));
 		if (id) {
-			nsString name;
-			name.AssignWithConversion(pName);
+      nsAutoString name, organization;
+      nsCOMPtr<nsIImportService> impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+      if (impSvc)
+      {
+        rv = impSvc->SystemStringToUnicode((const char *)pName, name);
+        if (NS_SUCCEEDED(rv))
+        {
 			id->SetFullName( name.get());
 			id->SetIdentityName( name.get());
+        }
+        rv = impSvc->SystemStringToUnicode((const char *)pOrgName, organization);
+        if (NS_SUCCEEDED(rv))
+          id->SetOrganization( organization.get());
+      }
 			id->SetEmail( pEmail);
 			if (pReply)
 				id->SetReplyTo( pReply);
@@ -484,6 +511,14 @@ void OutlookSettings::SetIdentities( nsIMsgAccountManager *pMgr, nsIMsgAccount *
 		}
 	}
 	
+  if (!pUserName) {
+    nsCOMPtr <nsIMsgIncomingServer>	incomingServer;
+    rv = pAcc->GetIncomingServer(getter_AddRefs( incomingServer));
+    if (NS_SUCCEEDED(rv) && incomingServer) 
+      rv = incomingServer->GetUsername(&pUserName);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to get UserName from incomingServer");
+  }
+
 	SetSmtpServer( pMgr, pAcc, pServer, pUserName);
 
 	nsOutlookRegUtil::FreeValueBytes( (BYTE *)pName);
