@@ -38,6 +38,7 @@
 #include "nsIPrefBranchInternal.h"
 #include "nsIFactory.h"
 #include "nsIComponentManager.h"
+#include "nsIObserver.h"
 #include "nsCOMPtr.h"
 #include "nsMemory.h"
 #include "prefapi.h"
@@ -51,7 +52,6 @@
 #include "nsISupportsPrimitives.h"
 #include "nsWeakReference.h"
 
-#include "nsIJSRuntimeService.h"
 #include "nsIModule.h"
 #include "nsIGenericFactory.h"
 
@@ -133,19 +133,13 @@ NS_IMPL_THREADSAFE_ISUPPORTS7(nsPref, nsIPref, nsIPrefService, nsIObserver, nsIP
 nsPref::nsPref()
 //----------------------------------------------------------------------------------------
 {
-  nsIPrefBranch *pBranch;
-  nsresult     rv;
-
   PR_AtomicIncrement(&g_InstanceCount);
-  NS_INIT_REFCNT();
 
-  nsCOMPtr<nsIPrefService> pService = 
-           do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
-  mPrefService = pService;
-
-  mPrefService->GetDefaultBranch("", &pBranch);
-  mDefaultBranch = (nsIPrefBranch *)pBranch;
+  mPrefService = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  NS_ASSERTION(mPrefService, "Preference Service failed to start up!!");
 
+  if (mPrefService)
+    mPrefService->GetDefaultBranch("", getter_AddRefs(mDefaultBranch));
 }
 
 //----------------------------------------------------------------------------------------
@@ -350,13 +344,13 @@ NS_IMETHODIMP nsPref::GetChildList(const char *aStartingAt, PRUint32 *aCount, ch
   return rv;
 }
 
-NS_IMETHODIMP nsPref::AddObserver(const char *aDomain, nsIObserver *aObserver)
+NS_IMETHODIMP nsPref::AddObserver(const char *aDomain, nsIObserver *aObserver, PRBool aHoldWeak)
 {
   nsresult rv;
 
   nsCOMPtr<nsIPrefBranchInternal> prefBranch = do_QueryInterface(mPrefService, &rv);
   if (NS_SUCCEEDED(rv))
-    rv = prefBranch->AddObserver(aDomain, aObserver);
+    rv = prefBranch->AddObserver(aDomain, aObserver, aHoldWeak);
   return rv;
 }
 
@@ -370,7 +364,7 @@ NS_IMETHODIMP nsPref::RemoveObserver(const char *aDomain, nsIObserver *aObserver
   return rv;
 }
 
-NS_IMETHODIMP nsPref::Observe(nsISupports *aSubject, const PRUnichar *aTopic, const PRUnichar *someData)
+NS_IMETHODIMP nsPref::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
 {
   nsresult rv;
 
@@ -382,7 +376,7 @@ NS_IMETHODIMP nsPref::Observe(nsISupports *aSubject, const PRUnichar *aTopic, co
 
 
 /*
- * Some temporary support for depricated functions.
+ * Some temporary support for deprecated functions.
  */
 
 /*
@@ -395,12 +389,13 @@ NS_IMETHODIMP nsPref::CopyUnicharPref(const char *pref, PRUnichar **_retval)
 
   nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(mPrefService, &rv);
   if (NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsISupportsWString> theString;
-    rv = prefBranch->GetComplexValue(pref, NS_GET_IID(nsISupportsWString),
+    nsCOMPtr<nsISupportsString> theString;
+    rv = prefBranch->GetComplexValue(pref, NS_GET_IID(nsISupportsString),
                                      getter_AddRefs(theString));
-    if (NS_SUCCEEDED(rv)) {
-      rv = theString->GetData(_retval);
-    }
+    if (NS_FAILED(rv))
+      return rv;
+
+    return theString->ToString(_retval);
   }
   return rv;
 }
@@ -408,14 +403,14 @@ NS_IMETHODIMP nsPref::CopyUnicharPref(const char *pref, PRUnichar **_retval)
 NS_IMETHODIMP nsPref::CopyDefaultUnicharPref(const char *pref, PRUnichar **_retval)
 {
   nsresult rv;
-  nsCOMPtr<nsISupportsWString> theString;
+  nsCOMPtr<nsISupportsString> theString;
 
-  rv = mDefaultBranch->GetComplexValue(pref, NS_GET_IID(nsISupportsWString),
+  rv = mDefaultBranch->GetComplexValue(pref, NS_GET_IID(nsISupportsString),
                                        getter_AddRefs(theString));
-  if (NS_SUCCEEDED(rv)) {
-    rv = theString->GetData(_retval);
-  }
-  return rv;
+  if (NS_FAILED(rv))
+    return rv;
+
+  return theString->ToString(_retval);
 }
 
 NS_IMETHODIMP nsPref::SetUnicharPref(const char *pref, const PRUnichar *value)
@@ -424,10 +419,10 @@ NS_IMETHODIMP nsPref::SetUnicharPref(const char *pref, const PRUnichar *value)
 
   nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(mPrefService, &rv);
   if (NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsISupportsWString> theString = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, &rv);
+    nsCOMPtr<nsISupportsString> theString = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv)) {
-      theString->SetData(value);
-      rv = prefBranch->SetComplexValue(pref, NS_GET_IID(nsISupportsWString), theString);
+      theString->SetData(nsDependentString(value));
+      rv = prefBranch->SetComplexValue(pref, NS_GET_IID(nsISupportsString), theString);
     }
   }
   return rv;
@@ -437,10 +432,10 @@ NS_IMETHODIMP nsPref::SetDefaultUnicharPref(const char *pref, const PRUnichar *v
 {
   nsresult rv;
 
-  nsCOMPtr<nsISupportsWString> theString = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, &rv);
+  nsCOMPtr<nsISupportsString> theString = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv)) {
-    theString->SetData(value);
-    rv = mDefaultBranch->SetComplexValue(pref, NS_GET_IID(nsISupportsWString), theString);
+    theString->SetData(nsDependentString(value));
+    rv = mDefaultBranch->SetComplexValue(pref, NS_GET_IID(nsISupportsString), theString);
   }
   return rv;
 }
@@ -455,7 +450,7 @@ NS_IMETHODIMP nsPref::GetLocalizedUnicharPref(const char *pref, PRUnichar **_ret
     rv = prefBranch->GetComplexValue(pref, NS_GET_IID(nsIPrefLocalizedString),
                                        getter_AddRefs(theString));
     if (NS_SUCCEEDED(rv)) {
-      rv = theString->GetData(_retval);
+      rv = theString->ToString(_retval);
     }
   }
   return rv;
@@ -469,8 +464,9 @@ NS_IMETHODIMP nsPref::GetDefaultLocalizedUnicharPref(const char *pref, PRUnichar
   rv = mDefaultBranch->GetComplexValue(pref, NS_GET_IID(nsIPrefLocalizedString),
                                        getter_AddRefs(theString));
   if (NS_SUCCEEDED(rv)) {
-    rv = theString->GetData(_retval);
+    rv = theString->ToString(_retval);
   }
+  
   return rv;
 }
 
@@ -630,7 +626,8 @@ NS_IMETHODIMP nsPref::CreateChildList(const char* parent_node, char **child_list
 NS_IMETHODIMP nsPref::NextChild(const char *child_list, PRInt16 *indx, char **listchild)
 {
   char* temp = (char*)&child_list[*indx];
-  char* child = strtok(temp, ";");
+  char* newstr;
+  char* child = nsCRT::strtok(temp, ";", &newstr);
   if (child) {
     *indx += PL_strlen(child) + 1;
     *listchild = child;
