@@ -40,6 +40,7 @@
 #include "nsPrefBranch.h"
 #include "nsILocalFile.h"
 #include "nsIObserverService.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIDirectoryService.h"
 #include "nsString.h"
@@ -112,8 +113,6 @@ static nsresult _convertRes(int res)
 nsPrefBranch::nsPrefBranch(const char *aPrefRoot, PRBool aDefaultBranch)
   : mObservers(nsnull)
 {
-  NS_INIT_ISUPPORTS();
-
   mPrefRoot = aPrefRoot;
   mPrefRootLength = mPrefRoot.Length();
   mIsDefault = aDefaultBranch;
@@ -373,13 +372,13 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
     return NS_OK;
   }
 
-  if (aType.Equals(NS_GET_IID(nsISupportsWString))) {
-    nsCOMPtr<nsISupportsWString> theString(do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, &rv));
+  if (aType.Equals(NS_GET_IID(nsISupportsString))) {
+    nsCOMPtr<nsISupportsString> theString(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv));
 
     if (NS_SUCCEEDED(rv)) {
-      rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String).get());
+      rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String));
       if (NS_SUCCEEDED(rv)) {
-        nsISupportsWString *temp = theString;
+        nsISupportsString *temp = theString;
 
         NS_ADDREF(temp);
         *_retval = (void *)temp;
@@ -462,13 +461,13 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
     return SetCharPref(aPrefName, descriptorString.get());
   }
 
-  if (aType.Equals(NS_GET_IID(nsISupportsWString))) {
-    nsCOMPtr<nsISupportsWString> theString = do_QueryInterface(aValue);
+  if (aType.Equals(NS_GET_IID(nsISupportsString))) {
+    nsCOMPtr<nsISupportsString> theString = do_QueryInterface(aValue);
 
     if (theString) {
-      nsXPIDLString wideString;
+      nsAutoString wideString;
 
-      rv = theString->GetData(getter_Copies(wideString));
+      rv = theString->GetData(wideString);
       if (NS_SUCCEEDED(rv)) {
         rv = SetCharPref(aPrefName, NS_ConvertUCS2toUTF8(wideString).get());
       }
@@ -594,7 +593,6 @@ NS_IMETHODIMP nsPrefBranch::GetChildList(const char *aStartingAt, PRUint32 *aCou
   char*           theElement;
   PRInt32         numPrefs;
   PRInt32         dwIndex;
-  nsresult        rv = NS_OK;
   EnumerateData   ed;
   nsAutoVoidArray prefArray;
 
@@ -602,6 +600,12 @@ NS_IMETHODIMP nsPrefBranch::GetChildList(const char *aStartingAt, PRUint32 *aCou
   NS_ENSURE_ARG_POINTER(aCount);
   NS_ENSURE_ARG_POINTER(aChildArray);
 
+  if (!gHashTable.ops) {
+    *aChildArray = nsnull;
+    *aCount = 0;
+    return NS_ERROR_NOT_INITIALIZED;
+  }
+
   // this will contain a list of all the pref name strings
   // allocate on the stack for speed
 
@@ -653,7 +657,6 @@ NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObser
   NS_ENSURE_ARG_POINTER(aObserver);
 
   if (!mObservers) {
-    nsresult rv = NS_OK;
     mObservers = new nsAutoVoidArray();
     if (nsnull == mObservers)
       return NS_ERROR_OUT_OF_MEMORY;
@@ -725,25 +728,25 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
 
       if (pCallback->pObserver == observerRef) {
         mObserverDomains.CStringAt(i, domain);
-        if (domain.Equals(aDomain))
-          break;
+        if (domain.Equals(aDomain)) {
+          // We must pass a fully qualified preference name to remove the callback
+          pref = getPrefName(aDomain); // aDomain == nsnull only possible failure, trapped above
+          rv = _convertRes(PREF_UnregisterCallback(pref, NotifyObserver, pCallback));
+          if (NS_SUCCEEDED(rv)) {
+            // Remove this observer from our array so that nobody else can remove
+            // what we're trying to remove ourselves right now.
+            mObservers->RemoveElementAt(i);
+            mObserverDomains.RemoveCStringAt(i);
+            NS_RELEASE(pCallback->pObserver);
+            nsMemory::Free(pCallback);
+          }
+          return rv;
+        }
       }
     }
   }
 
-  if (i == count)             // not found, just return
-    return NS_OK;
-
-  // We must pass a fully qualified preference name to remove the callback
-  pref = getPrefName(aDomain); // aDomain == nsnull only possible failure, trapped above
-  rv = _convertRes(PREF_UnregisterCallback(pref, NotifyObserver, pCallback));
-  if (NS_SUCCEEDED(rv)) {
-    NS_RELEASE(pCallback->pObserver);
-    nsMemory::Free(pCallback);
-    mObservers->RemoveElementAt(i);
-    mObserverDomains.RemoveCStringAt(i);
-  }
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
@@ -800,20 +803,22 @@ void nsPrefBranch::freeObserverList(void)
     if (count > 0) {
       PRInt32 i;
       nsCAutoString domain;
-      for (i = 0; i < count; i++) {
+      for (i = 0; i < count; ++i) {
         pCallback = (PrefCallbackData *)mObservers->ElementAt(i);
         if (pCallback) {
           mObserverDomains.CStringAt(i, domain);
           // We must pass a fully qualified preference name to remove the callback
           pref = getPrefName(domain.get()); // can't fail because domain must be valid
+          // Remove this observer from our array so that nobody else can remove
+          // what we're trying to remove right now.
+          mObservers->ReplaceElementAt(nsnull, i);
           PREF_UnregisterCallback(pref, NotifyObserver, pCallback);
           NS_RELEASE(pCallback->pObserver);
           nsMemory::Free(pCallback);
         }
       }
 
-      // now empty the observer arrays in bulk
-      mObservers->Clear();
+      // now empty the observer domains array in bulk
       mObserverDomains.Clear();
     }
     delete mObservers;
@@ -970,10 +975,7 @@ nsPrefLocalizedString::nsPrefLocalizedString()
 : mUnicodeString(nsnull)
 {
   nsresult rv;
-
-  NS_INIT_REFCNT();
-
-  mUnicodeString = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, &rv);
+  mUnicodeString = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
 }
 
 nsPrefLocalizedString::~nsPrefLocalizedString()
@@ -991,17 +993,46 @@ NS_IMPL_THREADSAFE_RELEASE(nsPrefLocalizedString)
 NS_INTERFACE_MAP_BEGIN(nsPrefLocalizedString)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPrefLocalizedString)
     NS_INTERFACE_MAP_ENTRY(nsIPrefLocalizedString)
-    NS_INTERFACE_MAP_ENTRY(nsISupportsWString)
+    NS_INTERFACE_MAP_ENTRY(nsISupportsString)
 NS_INTERFACE_MAP_END
 
 nsresult nsPrefLocalizedString::Init()
 {
   nsresult rv;
-  mUnicodeString = do_CreateInstance(NS_SUPPORTS_WSTRING_CONTRACTID, &rv);
+  mUnicodeString = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv);
 
   return rv;
 }
 
+NS_IMETHODIMP
+nsPrefLocalizedString::GetData(PRUnichar** _retval)
+{
+  nsAutoString data;
+
+  nsresult rv = GetData(data);
+  if (NS_FAILED(rv))
+    return rv;
+  
+  *_retval = ToNewUnicode(data);
+  if (!*_retval)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPrefLocalizedString::SetData(const PRUnichar *aData)
+{
+  return SetData(nsDependentString(aData));
+}
+
+NS_IMETHODIMP
+nsPrefLocalizedString::SetDataWithLength(PRUint32 aLength,
+                                         const PRUnichar* aData)
+{
+  return SetData(Substring(aData, aData + aLength));
+}
+
 //----------------------------------------------------------------------------
 // nsRelativeFilePref
 //----------------------------------------------------------------------------
@@ -1010,7 +1041,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsRelativeFilePref, nsIRelativeFilePref);
 
 nsRelativeFilePref::nsRelativeFilePref()
 {
-    NS_INIT_ISUPPORTS();
 }
 
 nsRelativeFilePref::~nsRelativeFilePref()
