@@ -28,6 +28,7 @@
 #include "nsHttpDigestAuth.h"
 #include "nsIHttpChannel.h"
 #include "nsIServiceManager.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIURI.h"
 #include "nsString.h"
@@ -44,8 +45,6 @@
 
 nsHttpDigestAuth::nsHttpDigestAuth()
 {
-  NS_INIT_ISUPPORTS();
-
   mVerifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID);
   mGotVerifier = (mVerifier != nsnull);
 
@@ -112,6 +111,14 @@ nsHttpDigestAuth::GenerateCredentials(nsIHttpChannel *httpChannel,
   PRBool isDigestAuth = !PL_strncasecmp(challenge, "digest ", 7);
   NS_ENSURE_TRUE(isDigestAuth, NS_ERROR_UNEXPECTED);
 
+  // IIS implementation requires extra quotes
+  PRBool requireExtraQuotes = PR_FALSE;
+  nsCAutoString serverVal;
+  httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Server"), serverVal);
+  if (!serverVal.IsEmpty()) {
+    requireExtraQuotes = !PL_strncasecmp(serverVal.get(), "Microsoft-IIS", 13);
+  }
+
   NS_ConvertUCS2toUTF8 cUser(username), cPass(password);
 
   nsresult rv;
@@ -160,7 +167,9 @@ nsHttpDigestAuth::GenerateCredentials(nsIHttpChannel *httpChannel,
     if (http_channel != nsnull)
     {
       nsIInputStream * upload;
-      http_channel->GetUploadStream(&upload);
+      nsCOMPtr<nsIUploadChannel> uc = do_QueryInterface(http_channel);
+      NS_ENSURE_TRUE(uc, NS_ERROR_UNEXPECTED);
+      uc->GetUploadStream(&upload);
       if (upload) {
         char * upload_buffer;
         int upload_buffer_length = 0;
@@ -250,10 +259,14 @@ nsHttpDigestAuth::GenerateCredentials(nsIHttpChannel *httpChannel,
 
   if (qop) {
     authString += "\", qop=";
+    if (requireExtraQuotes)
+      authString += "\"";
     if (qop & QOP_AUTH_INT)
       authString += "auth-int";
     else
       authString += "auth";
+    if (requireExtraQuotes)
+      authString += "\"";
     authString += ", nc=";
     authString += nonce_count;
     authString += ", cnonce=\"";
@@ -272,6 +285,24 @@ nsHttpDigestAuth::AreCredentialsReusable(PRBool *result)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsHttpDigestAuth::ChallengeRequiresUserPass(const char *challenge,
+                                            PRBool *result)
+{
+  nsCAutoString realm, domain, nonce, opaque;
+  PRBool stale;
+  PRUint16 algorithm, qop;
+
+  nsresult rv = ParseChallenge(challenge, realm, domain, nonce, opaque,
+                               &stale, &algorithm, &qop);
+  if (NS_FAILED(rv)) {
+    *result = PR_TRUE;
+    return rv;
+  }
+  *result = !stale;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsHttpDigestAuth::AllocateMetaData(nsISupports **result)
 {
