@@ -1,26 +1,40 @@
-/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   John C. Griggs <johng@corel.com>
  *
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsCOMPtr.h"
 #include "nsAppShell.h"
@@ -43,19 +57,15 @@ PRInt32 gAppShellID = 0;
 static NS_DEFINE_IID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 static NS_DEFINE_IID(kCmdLineServiceCID, NS_COMMANDLINE_SERVICE_CID);
 
-PRBool nsAppShell::mRunning = PR_FALSE;
-
 //-------------------------------------------------------------------------
 //
 // nsAppShell constructor
 //
 //-------------------------------------------------------------------------
-nsAppShell::nsAppShell()
+nsAppShell::nsAppShell() :
+  mDispatchListener(0),
+  mApplication(nsnull)
 {
-  NS_INIT_REFCNT();
-  mDispatchListener = 0;
-  mEventQueue = nsnull;
-  mApplication = nsnull;
 #ifdef DBG_JCG
   gAppShellCount++;
   mID = gAppShellID++;
@@ -200,9 +210,7 @@ NS_METHOD nsAppShell::Run()
   if (!mEventQueue) {
     return NS_ERROR_NOT_INITIALIZED;
   }
-  mRunning = PR_TRUE;
   mApplication->exec();
-  mRunning = PR_FALSE;
 
   Spindown();
   return NS_OK;
@@ -219,8 +227,7 @@ NS_METHOD nsAppShell::Exit()
   if (mApplication == nsnull)
     return NS_ERROR_NOT_INITIALIZED;
 
-  if (mRunning)
-    mApplication->exit(0);
+  mApplication->exit(0);
   return NS_OK;
 }
 
@@ -236,9 +243,6 @@ NS_METHOD nsAppShell::DispatchNativeEvent(PRBool aRealEvent, void *aEvent)
   if (mApplication == nsnull)
     return NS_ERROR_NOT_INITIALIZED;
 
-  if (!mRunning)
-    return NS_ERROR_NOT_INITIALIZED;
-
   mApplication->processEvents(1);
   return NS_OK;
 }
