@@ -346,7 +346,6 @@ public:
   nsCharsetMenuObserver(nsCharsetMenu * menu)
     : mCharsetMenu(menu)
   {
-    NS_INIT_ISUPPORTS();
   }
 
   virtual ~nsCharsetMenuObserver() {}
@@ -445,7 +444,6 @@ nsCharsetMenu::nsCharsetMenu()
   mAutoDetectInitialized(PR_FALSE),
   mOthersInitialized(PR_FALSE)
 {
-  NS_INIT_REFCNT();
   NS_TIMELINE_START_TIMER("nsCharsetMenu::nsCharsetMenu");
   nsresult res = NS_OK;
 
@@ -759,12 +757,7 @@ nsresult nsCharsetMenu::FreeResources()
       pbi->RemoveObserver(kBrowserStaticPrefKey, mCharsetMenuObserver);
       pbi->RemoveObserver(kMaileditPrefKey, mCharsetMenuObserver);
     }
-    nsCOMPtr<nsIObserverService> observerService = 
-         do_GetService("@mozilla.org/observer-service;1", &res);
-
-    if (NS_SUCCEEDED(res))
-      res = observerService->RemoveObserver(mCharsetMenuObserver, 
-                                            "charsetmenu-selected");
+    /* nsIObserverService has to have released nsCharsetMenu already */
   }
 
   mRDFService = NULL;
@@ -1530,7 +1523,7 @@ nsresult nsCharsetMenu::UpdateCachePrefs(const char * aCacheKey,
   nsresult res = NS_OK;
   char * cachePrefValue = NULL;
   char * staticPrefValue = NULL;
-  const char * currentCharset = NS_ConvertUCS2toUTF8(aCharset).get();
+  NS_ConvertUCS2toUTF8 currentCharset(aCharset);
   PRInt32 cacheSize = 0;
 
   res = mPrefs->GetCharPref(aCacheKey, &cachePrefValue);
@@ -1962,14 +1955,6 @@ NS_IMETHODIMP nsCharsetMenu::GetAllResources(nsISimpleEnumerator** aCursor)
   return mInner->GetAllResources(aCursor);
 }
 
-NS_IMETHODIMP nsCharsetMenu::GetAllCommands(
-                             nsIRDFResource* source,
-                             nsIEnumerator/*<nsIRDFResource>*/** commands)
-{
-  NS_NOTYETIMPLEMENTED("write me!");
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 NS_IMETHODIMP nsCharsetMenu::GetAllCmds(
                              nsIRDFResource* source,
                              nsISimpleEnumerator/*<nsIRDFResource>*/** commands)
